/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       changedir.cpp                                             *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       13-Apr-2013 18.51.25 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* File added.                                                               *
* Purpose is to change to a different project directory.                    *
* It can also create a new directory.                                       *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/



#include "misc.h"
#include "changedir.h"

GtkWidget *ChangeDir::directory_chooser = NULL;

using namespace misc;

ChangeDir::ChangeDir(RevHead *revhead)
{
cout << "ChangeDir:" << endl;
rh = revhead;
}

ChangeDir::~ChangeDir(void)
{

cout << "~ChangeDir:" << endl;
}

void ChangeDir::callback_cancel_change_directory(GtkWidget *widget, ChangeDir *cd)
{
cout << "ChangeDir::callback_cancel_change_directory" << endl;
if (cd->dirname_only)
	{
	cout << "ChangeDir::callback_cancel_change_directory- cd->dirname_only=TRUE" << endl;
	gtk_main_quit();
	return;
//	gtk_widget_show_all(cd->rh->vbox);
	}

cd->rh->FrontPanel();
delete cd;
}


string ChangeDir::get_dirname(void)
{
return dirname;
}

void ChangeDir::callback_do_change_dir(GtkWidget *widget, ChangeDir *cd)
{
string buffer;

	
gchar *path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(cd->directory_chooser));
	
if (path) cd->dirname = path;
else return;

g_free(path);

if (cd->dirname_only)
	{

	cout << "ChangeDir::callback_do_change_dir - cd->dirname_only TRUE" << endl;
	gtk_main_quit();	
	return;
	}

if (!cd->dirname.empty())
	{
	if (!STREQ(cd->dirname.c_str(), (char *)cd->rh->path.c_str() )) // If not still in the same directory
		{
		if (chdir(cd->dirname.c_str()))
			{
			// Failed to chdir
			buffer = "Can't change directory from \"" + cd->rh->path;
			buffer += "\" to \"";
			buffer += cd->dirname;
			buffer += '\"';
			cd->rh->logfile->report(buffer);

			cout << "ChangeDir::do_change_dir - " << buffer << endl;
			} 
		else
			{
			cout << "ChangeDir::do_change_dir - SUCCESS" << endl;
			if (cd->rh->logfile->changed)
				{
				// Keep old log
				}
			else
				{
				// Delete old log
				cd->rh->logfile->delete_logfile();
				}

			// Fix up the local info as if a new start
  		//========================================== 
  		cd->rh->logfile->close();

cout << "ChangeDir::callback_do_change_directory - CLOSED LOGFILE" << endl;
  
			// Initialize (zero) all data
			// Zero all the flags
        

      // Collect info if there is any
  
      cd->rh->flags.rh_starting = 1;

	
			gtk_widget_destroy (cd->directory_chooser);
			cd->directory_chooser = NULL;
			cd->rh->FrontPanel();
			delete cd;

      // See if RH_ChangeLog exists to correct this maybe
      // In fact, get_env could do this and set or clear accordingly
      //if (rh->flags.newproject) // This has just been cleared!
      //  {
      //  rh->ev = new Environment(rh);
      //  rh->ev->get_env(FALSE);
      //  }
      
      }
    }
  else
    {
    cout << "           SAME PATHNAME!" << endl;
    }
  }
else cout << "           FAILED TO GET PATHNAME!" << endl;
}





void ChangeDir::change_dir(string title, bool dirname_only)
{
GtkWidget *wid, *hbox;
	
this->dirname_only = dirname_only;
	
rh->DestroyWidgets();
dirname.clear();
//cout << "ChangeDir::change_dir - title= " << title << endl;
	
wid = gtk_label_new(title.c_str());
gtk_container_add(GTK_CONTAINER(rh->vbox), wid);

	
directory_chooser = gtk_file_chooser_widget_new( GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER );
gtk_box_pack_start (GTK_BOX(rh->vbox), directory_chooser, TRUE, TRUE, 0);
	
gtk_file_chooser_set_create_folders (GTK_FILE_CHOOSER (directory_chooser), TRUE);


hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER(directory_chooser), hbox);


wid = gtk_button_new_with_label(RevHead::CancelTXT.c_str());
g_signal_connect (G_OBJECT (wid), "clicked",
                        G_CALLBACK(callback_cancel_change_directory), this);
gtk_box_pack_start(GTK_BOX(hbox), wid, TRUE, TRUE, 0);
	
wid = gtk_button_new_with_label((char *)RevHead::OKTXT.c_str());
g_signal_connect (G_OBJECT (wid), "clicked",
            G_CALLBACK(callback_do_change_dir), this);
gtk_box_pack_start(GTK_BOX(hbox), wid, TRUE, TRUE, 0);

gtk_widget_show_all(rh->vbox);

}
