/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       complain.cpp                                              *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       15-Apr-2013 18.37.39 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Bring up a "complain" window to inform the user of a problem etc.         *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "complain.h"

using namespace std;


void Complain::end_complain (GtkWidget *widget, Complain *c)
{
gtk_widget_destroy (c->dialog_window);
delete(c);
}


void Complain::clear_complain(GtkWidget *widget, Complain *c)
{
gtk_grab_remove (widget);
}

void Complain::initialize(GdkPixbuf *pixbuf, 
                    GdkRGBA *bg_normal, GdkRGBA *bg_prelight,
                    const gchar *cancel_stock_id, const gchar *ok_stock_id)
{
image_pixbuf = pixbuf;
this->bg_normal       = bg_normal;
this->bg_prelight     = bg_prelight;
this->cancel_stock_id = cancel_stock_id;
this->ok_stock_id     = ok_stock_id;
dialog_window = NULL;
}

// This has to be called using "new" so that it remains in memory
// end_complain calls "delete"
Complain::Complain(void)
{
cout << "Complain:\n";
initialize(NULL, NULL, NULL, NULL, NULL); 
}

Complain::Complain(GdkPixbuf *pixbuf)
{
initialize(pixbuf, NULL, NULL, NULL, NULL); 
}

Complain::Complain(GdkPixbuf *pixbuf, 
                    GdkRGBA *bg_normal, GdkRGBA *bg_prelight,
                    const gchar *cancel_stock_id, const gchar *ok_stock_id)
{
cout << "Complain:\n";
initialize(pixbuf, bg_normal, bg_prelight, cancel_stock_id, ok_stock_id); 
}



/***************************************
 * Colourize a widget                  *
 * fg bg prelight stock_id             *
 * Do nothing for NULL colour pointers *
 ***************************************/
void Complain::decorate_widget(GtkWidget *widget, GdkRGBA *fg_normal, GdkRGBA *bg_normal,  GdkRGBA *bg_prelight, const gchar *stock_id)
{
if (fg_normal) gtk_widget_override_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, fg_normal); 
if (bg_normal) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, bg_normal);  
if (bg_prelight) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_PRELIGHT, bg_prelight);

if (GTK_IS_BUTTON(widget) && (stock_id) && *(stock_id))
  {
  gtk_button_set_image(GTK_BUTTON(widget), gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON));
  // Make sure the image is displayed!
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(widget)))); 
  }
}

Complain::~Complain(void)
{
cout << "~Complain:\n";
}

/************************************************
 * Complain(n, title, msg)                      *
 * Displays a requester.                        *
 * (n = 1) gives OK or (n = 2) RETRY and ABORT. *
 * or (n=-1) gives only CONTINUE gadget.        *
 * (n = 0) gives only ABORT gadget.             *
 * Returns TRUE (OK gadget) or FALSE.           *
 ***********************************************/

// Not sure how to make this return a value
// I think that to make this a general purpose subroutine
// it has to have the left & right functions passed as well.
// What happens about their args?
// This prevents serial programming!  The calling code carries on
// and doesn't stop as far as I can see.
// This code should not only take the focus; it also has to grab
// the execution completely from the calling routine without
// returning until it exits with a mouse button or whatever.

void Complain::complain (int n, char *title, char *msg, 
                        void (*YesFunc)(GtkWidget *widget, Complain *c), 
                        void (*NoFunc)(GtkWidget *widget, Complain *c),
                        gpointer d)
{
GtkWidget *label1, *label2;
GtkWidget *button;
const char *lb, *rb;
gint x, y, w, h;
gint sh, sw;
GdkScreen *screen;
data = d;
screen = gtk_window_get_screen(GTK_WINDOW(RevHead::window));
sh = gdk_screen_get_height(screen);
sw = gdk_screen_get_width(screen);


switch (n)
    {
    case CN_ABORT:
        lb = "ABORT";
        rb = NULL;
    break;

    case CN_CONTINUE:
        lb = NULL;
        rb = "CONTINUE";
    break;

    case CN_OK:
        lb = NULL;
        rb = "OK";
    break;
 
    case CN_CANCEL:
        lb = "CANCEL";
        rb = NULL;
    break;
 
    case CN_OK_CANCEL:
        lb = "CANCEL";
        rb = "OK";
    break;

    case CN_OK_ABORT:
        lb = "ABORT";
        rb = "OK";
    break;

    case CN_RETRY_ABORT:
        lb = "ABORT";
        rb = "RETRY";
    break;

    case CN_RETRY_CANCEL:
        lb = "CANCEL";
        rb = "RETRY";
    break;

    case CN_YES_NO:
    default:
        lb = "NO";
        rb = "YES";
    break;
    }

/* --- Create a dialog window --- */
dialog_window = gtk_dialog_new ();

 
g_signal_connect (G_OBJECT (dialog_window), "destroy",
              G_CALLBACK(clear_complain),
              this);

/* --- Set the title and add a border --- */
gtk_window_set_title (GTK_WINDOW (dialog_window), title);
//gtk_container_border_width (GTK_CONTAINER (dialog_window), 0);
gtk_container_set_border_width(GTK_CONTAINER(dialog_window), 0);

  
// Place the requester in middle of the panel window
gtk_window_get_position( GTK_WINDOW(RevHead::window), &x, &y);

gtk_window_get_size( GTK_WINDOW(RevHead::window), &w, &h);

gtk_widget_set_size_request(GTK_WIDGET(dialog_window), 3*RevHead::width/4, RevHead::height/2); // width, height


x += w/2-RevHead::width/3;
y += (h-RevHead::height/3)/2;

gtk_window_move(GTK_WINDOW(dialog_window), x, y);

/* --- Create an LEFT button (with the focus if no right button) --- */
if (lb)
    {
    button = gtk_button_new_with_label ( lb );
//    decorate_button(button, bg_normal, bg_prelight, cancel_stock_id);

		decorate_widget(button, NULL, bg_normal, bg_prelight, cancel_stock_id);

    if (NoFunc)
        {
        g_signal_connect (G_OBJECT (button), "clicked",
                            G_CALLBACK(NoFunc),
                            this);
        }
    else
        {
        g_signal_connect (G_OBJECT (button), "clicked",
                            G_CALLBACK(end_complain),
                            this);
        }

    /* --- Default the "Ok" button --- */
    //gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
    //                        button, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG(dialog_window))), 
              button, TRUE, TRUE, 0);
	
			
    if(!rb)
        {
        //GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
				gtk_widget_set_can_default(GTK_WIDGET(button), TRUE);
        gtk_widget_grab_default (button);
        }
    }

/* --- Create a RIGHT button with the focus --- */
if (rb)
  {
  button = gtk_button_new_with_label ( rb );
//  decorate_button(button, bg_normal, bg_prelight, ok_stock_id);

 	decorate_widget(button, NULL, bg_normal, bg_prelight, ok_stock_id);
  if (YesFunc)
    {
	  g_signal_connect (G_OBJECT (button), "clicked",
				G_CALLBACK(YesFunc),
				this);
    }
  else
    {
	  g_signal_connect (G_OBJECT (button), "clicked",
			    G_CALLBACK(end_complain),
			    this);
    }

//   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
		gtk_widget_set_can_default(GTK_WIDGET(button), TRUE);
		
//        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
//              button, TRUE, TRUE, 0);


		gtk_box_pack_start(GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG(dialog_window))), 
			    button, TRUE, TRUE, 0);

		
    gtk_widget_grab_default (button);
  }
string m = msg;
string msg1;
string msg2;
unsigned int i;
if (msg)
  {
  for (i=0; i < m.length(); ++i)
    {
    if (m[i] == '\n')
      {
      i++;
      break;
      }
    msg1 += m[i];
    }

  if (m.length() > i)
    {
    msg2 = m.substr(i);
    }
  }
  
  
label1 = gtk_label_new (msg1.c_str());
gtk_misc_set_padding (GTK_MISC (label1), 2, 2);
if (msg2.length())
  { 
  label2 = gtk_label_new (msg2.c_str());
  gtk_misc_set_padding (GTK_MISC (label2), 2, 2);
  } 
else label2 = NULL;
  
if (image_pixbuf)
  {

GtkWidget *grid = gtk_grid_new();
gtk_grid_set_row_homogeneous(GTK_GRID(grid), TRUE);
gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
gtk_widget_show_all(grid);
		

	gtk_container_add(GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG(dialog_window))), grid);
      
//  GdkPixbuf *pb = gdk_pixbuf_scale_simple(image_pixbuf, h/12, h/12, GDK_INTERP_BILINEAR);
  GdkPixbuf *pb = gdk_pixbuf_scale_simple(image_pixbuf, h/8, h/8, GDK_INTERP_BILINEAR);


	gtk_grid_attach(GTK_GRID(grid),  
									gtk_image_new_from_pixbuf(pb),
	                0, 0, 1, 1);  // left, top, width, height
		


	gtk_grid_attach(GTK_GRID(grid),
	                label1,
	                1, 0, 4, 1);  // left, width, top, height


		
  if (label2)
    {

    gtk_container_add(GTK_CONTAINER (gtk_dialog_get_content_area(GTK_DIALOG (dialog_window))), label2);

			
    }
  }
else  
  {

	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dialog_window))), 
              label1, TRUE, TRUE, 0);


		
  if (label2)
    {
	 	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dialog_window))),
		                     label1, TRUE, TRUE, 0);

   }
  }

// Make sure the dialog keeps the focus
gtk_grab_add(dialog_window);
gtk_widget_grab_focus(dialog_window);

gtk_widget_show_all(dialog_window);
}

