/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       environment.cpp                                           *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Apr-2013 09.56.02 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Split off from elsewhere.                                                 *
* Very similar in scheme to previous version.                               *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <list>

#include "environment.h"

#include "complain.h"
#include "date.h"
#include "entrylist.h"
#include "logfile.h"
#include "getproject.h"
#include "tfile.h"
#include "misc.h"
#include "RH_version.h"

using namespace std;
using namespace misc;

string Environment::Env_FileTXT;

GtkWidget *Environment::treeview = NULL;
string Environment::RH_runTXT    = "RH_run";

    
Environment::Environment(RevHead *revhead)
{
cout << "Environment:" << endl;

rh = revhead;
Env_FileTXT = rh->project_path + "/.revhead.project";
existing_project_flag = FALSE;

// Zero the widgets
p_want_micro_checkbutton =
p_auto_inc_checkbutton =

c_file_widget =
c_c_type_checkbox =
c_version_checkbox =
    
cpp_file_widget =
cpp_c_type_checkbox =
cpp_cpp_type_checkbox =

ada_file_widget =
ada_type_checkbox =

php_file_widget =
php_c_type_checkbox =
php_cpp_type_checkbox =
php_cpp2_type_checkbox =
php_hash_type_checkbox =

java_file_widget =
java_c_type_checkbox =
java_cpp_type_checkbox =
java_cpp2_type_checkbox =
  
none_log_checkbox =
append_log_checkbox =
overwrite_log_checkbox = 
c_path_entry = c_clear_button = c_browse_button = 
ada_path_entry = ada_clear_button = ada_browse_button = 
php_path_entry = php_clear_button = php_browse_button = 
java_path_entry = java_clear_button = java_browse_button = 
perl_path_entry = perl_clear_button = perl_browse_button = 
fortran_path_entry = fortran_clear_button = fortran_browse_button = NULL;
}

Environment::~Environment(void)
{

cout << "~Environment:" << endl;

if (gtk_main_level() > 1) gtk_main_quit();
}


// date owner email name version summary comment;
list<string>::iterator Environment::insert_date_pname_version_comment(list<string>::iterator &i, bool overwrite)
{
Date         dt(rh);
string       buffer;

int daylight = dt.get_daylight_saving(rh->p_currentds, rh->p_currenttz);

// First the date in the form of yyyy-mm-dd hh:mm:ss tz daylight
dt.ds2date(&rh->p_currentds, &rh->p_currenttz, &daylight, &buffer,Y_M_D_T);
buffer += "\t" + rh->p_owner;

// Next after that should be that person's email address
UNLESS (rh->p_owner_email.empty())
  {
  buffer += "\t<" + rh->p_owner_email + ">";
  }
  
if (overwrite)
  {
  (*i) = buffer;
  ++i;
  }
else
  {
  rh->changelog.insert(i, buffer);
  }


if (overwrite) ++i;
else rh->changelog.insert(i,"\n");


buffer = "\t* " + rh->get_project_name_and_version();

//cout << "Environment::insert_date_pname_version_comment" << endl; 
//cout << "    buffer= \"" << buffer << "\"" << endl;
  
if (overwrite)
  {
  (*i) = buffer;
  ++i;
  }
else rh->changelog.insert(i, buffer);



if ((rh->flags.newsummary) && (!rh->p_summary.empty()))
  {
  list<string>::iterator s;

  rh->changelog.insert(i, "\t  New Summary:-");

  for (s=rh->p_summary.begin(); s!=rh->p_summary.end(); ++s)
    {
    buffer = "\t    " + (*s);
    rh->changelog.insert(i, buffer);
    }
  }

if (!rh->p_comment.empty())
  {
  list<string>::iterator c;
  string indent = "";
  if ((rh->flags.newsummary) && (!rh->p_summary.empty()))
    {
    rh->changelog.insert(i, "\t  Comment:-");
    indent = "  ";
    }
  for (c=rh->p_comment.begin(); c!=rh->p_comment.end(); ++c)
    {
    buffer = "\t  " + indent + (*c);
    rh->changelog.insert(i, buffer);
    }
  }

return i;
}


void Environment::update_changelog(void)
{
Date    dt(rh);
string buffer;
list<string>::iterator i_start;
list<string>::iterator i;
  
bool failed = TRUE;  // Assume this


if (!rh->flags.newproject)
  if (!rh->flags.newname)
    if (!rh->flags.new_major_version)
      if (!rh->flags.new_minor_version)
        if (!rh->flags.new_micro_version)
  {

// cout << "Environment::update_existing_changelog - same name and version" << endl;

  i = rh->changelog.begin();
  if (i!=rh->changelog.end())
    {
    if ((*i).find(rh->p_owner))
      {
      i_start = i;
      i++;
      if (i!=rh->changelog.end())
        {
        if ((*i) == "\n")
          {
          i++;
          if (i!=rh->changelog.end())
            {
            if ((*i).find( rh->get_project_name_and_version() ))
              {
              // GOT same name and version - DO IT HERE
//  cout << "Environment::update_existing_changelog - << GOT SAME ENTRY >>" << endl;

              i = insert_date_pname_version_comment(i_start, TRUE);
                  
              while (i!=rh->changelog.end()) // get end of this paragraph
                {
                if ( (*i) == "\n")
                  {
                  i++;
                  break;
                  }
                else i++;               
                }
              failed = FALSE;


              }
            }
          }
        }
      }
    }
  }


if (failed)         // either not same name version etc OR a new file
  {
//cout << "Environment::update_existing_changelog - << NOT SAME ENTRY >>" << endl;
  i = rh->changelog.begin();
//  i++;
  insert_date_pname_version_comment(i, FALSE);
  rh->changelog.insert(i, "\n");
  }


list<Files>::iterator  f;
  
// Insert changed file details
for (f=rh->file_list.begin(); f!=rh->file_list.end(); ++f)
  {
  if (!(*f).flags.changed) continue;

// filename, date, author, email
  dt.ds2date(&(*f).datestamp, &(*f).timezone, &(*f).daylight, &buffer, Y_M_D_T);

  buffer = "\t* " + (*f).name + "\t" + buffer;
    
  UNLESS ((*f).author.empty())
    {
    buffer += "\t" + (*f).author;
    }
  UNLESS ((*f).author_email.empty())
    {
    buffer += "\t<" + (*f).author_email + ">";
    }

  rh->changelog.insert(i, buffer);

//  if (rh->flags.f_versions)      
  if ((*f).flags.version_wanted)
    {
    buffer = "\t  File Version ";
    rh->append_version(buffer, (*f).major_version, (*f).minor_version, (*f).micro_version);

    rh->changelog.insert(i, buffer);
    }

  if (!rh->source_path.empty())
    {
    list<string>::iterator s;
    int n = 0;
    for (s = rh->source_path.begin(); s != rh->source_path.end(); ++s)
      {
      n++;
      }
    if (n>1)
      {
      string path = (*f).pathname.substr(0,  ((*f).pathname.length() - (*f).name.length()) );
      buffer = "\t  Directory \"" + path + "\"";
      rh->changelog.insert(i, buffer);
      }
    }

// If there is a new file summary
  if ((*f).flags.newsummary && (!(*f).summary.empty()))
    {
    list<string>::iterator s;

    rh->changelog.insert(i, "\t  New File Summary:-");
 
    for (s=(*f).summary.begin(); s!=(*f).summary.end(); ++s)
      {
      buffer = "\t    " + (*s);
      rh->changelog.insert(i, buffer);    
      }
    }

// File comments if any
  if (!(*f).comment.empty())
    {
    list<string>::iterator c;

    string indent = "";
    if (((*f).flags.newsummary) && (!(*f).summary.empty()))
      {                
      rh->changelog.insert(i, "\t  Comment:-");
      indent = "  ";
      }
 
    for (c=(*f).comment.begin(); c!=(*f).comment.end(); ++c)
      {
      buffer = "\t  " + indent + (*c);
      rh->changelog.insert(i, buffer);
      }
    }

  rh->changelog.insert(i, "\n");
  
  }    // END of for

// The rest of changelog is all the same...

save_environment(this);
}



void Environment::save_environment(Environment *ev)
{
TFile  tf;
fstream ofp;
string buffer;
string msg;
string temp_env;
string *tname = tf.tfile( &Env_FileTXT, &ofp, 3);   // Open temp for write

if (tname)
  {
  Date dt(rh);
  list<string>::iterator i;
  string temp;
  temp_env = *tname;
  msg = "Writing file \"" + Env_FileTXT + "\"";
  msg += "\n    Opened \"" + temp_env + "\" for write";
  ev->rh->logfile->report(msg);

  buffer = "# Environment file \"" +  Env_FileTXT + "\"\n# created by " + RH_PROJECT_NAME + "-" + RH_PROJECT_VERSION + " "  + dt.get_date_now() + "\n";

// Project
  buffer += "\n# Project Details\n" + P_NameTXT + "    = " + ev->rh->p_name + "\n";
  buffer += P_VersionTXT + " = ";
  ev->rh->append_version(buffer, ev->rh->p_major_version, ev->rh->p_minor_version, ev->rh->p_micro_version);        

  buffer += "\n" + P_EmailTXT + "   = " + ev->rh->p_owner_email + "\n";

  if (ev->rh->flags.newproject) 
    {
		// NEW PROJECT
    Date dt(rh);
    dt.get_current_time();  // May not be from Update(), so make sure we have it
    gtk_label_set_text (GTK_LABEL (rh->title), (char *)RevHead::NewProjectTXT.c_str() );
    rh->p_startds = rh->p_currentds;
    rh->p_starttz = rh->p_currenttz;
    rh->p_start_daylight = rh->p_current_daylight;
    }
    
// Start Datestamp
    string datebuffer;
    dt.ds2date(&ev->rh->p_startds, &ev->rh->p_starttz, &ev->rh->p_start_daylight, &datebuffer, D_M_Y_T);
    buffer += P_StartedTXT + "    = " + datebuffer + "\n";   


// Summary
    if (!ev->rh->p_summary.empty())
      {
      buffer += Start_SummaryTXT + "\n";
      for (i=ev->rh->p_summary.begin(); i!=ev->rh->p_summary.end(); ++i)
        {
        buffer += "    " + (*i) + "\n";
        }
      buffer += End_SummaryTXT + "\n";
      }


// Contact
  if (!ev->rh->p_address.empty())
    {
    buffer += Start_ContactTXT + "\n";
    for (i=ev->rh->p_address.begin(); i!=ev->rh->p_address.end(); ++i)
      {
      buffer += "    " + (*i) + "\n";
      }
    buffer += End_ContactTXT + "\n";
    }

// Copyright
  buffer += "\n# Copyright Details\nCopyright_Owner = " + ev->rh->p_owner + "\n";

  if (!ev->rh->p_copyright.empty())
    {
    buffer += Start_CopyrightTXT + "\n";
    for (i=ev->rh->p_copyright.begin(); i!=ev->rh->p_copyright.end(); ++i)
      {
      buffer += "    " + (*i) + "\n";
      }
    buffer += End_CopyrightTXT + "\n";
    }

// Languages, Version & Log
  buffer += "\n# Languages, Versions and Log\n";
    
  // Languages
  buffer += (ev->rh->flags.cpp_file)     ? "CPP                    = 1\n" : "";
  buffer += (ev->rh->flags.c_file)       ? "C                      = 1\n" : "";         
  buffer += (ev->rh->flags.cpp_t)        ? "CC_Type1               = 1\n" : "";
  buffer += (ev->rh->flags.cpp2_t)       ? "CC_Type2               = 1\n" : "";
  buffer += (ev->rh->flags.c_v)          ? "C_Version              = 1\n" : "";

  buffer += (ev->rh->flags.ada_file)     ? "ADA                    = 1\n" : "";
  buffer += (ev->rh->flags.ada_v)        ? "ADA_Version            = 1\n" : "";

  buffer += (ev->rh->flags.php_file)     ? "PHP                    = 1\n" : "";
  if (ev->rh->flags.php_c_t)             buffer += "PHP_C_Type             = 1\n"; // Only one of these
  else if (ev->rh->flags.php_cpp_t)      buffer += "PHP_Type1              = 1\n";
  else if (ev->rh->flags.php_cpp2_t)     buffer += "PHP_Type2              = 1\n";
  else if (ev->rh->flags.php_h_t)        buffer += "PHP_Hash_Type          = 1\n";
  buffer += (ev->rh->flags.php_v)        ? "PHP_Version            = 1\n" : "";

  buffer += (ev->rh->flags.java_file)    ? "JAVA                   = 1\n" : "";
  if (ev->rh->flags.java_c_t)            buffer += "JAVA_C_Type            = 1\n"; // Only one of these
  else if (ev->rh->flags.java_cpp_t)     buffer += "JAVA_CC_Type1          = 1\n";
  else if (ev->rh->flags.java_cpp2_t)    buffer += "JAVA_CC_Type2          = 1\n";
  buffer += (ev->rh->flags.java_v)       ? "JAVA_Version           = 1\n" : "";

  buffer += (ev->rh->flags.perl_file)    ? "PERL                   = 1\n" : "";
  buffer += (ev->rh->flags.perl_v)       ? "PERL_Version           = 1\n" : "";

  buffer += (ev->rh->flags.fortran_file) ? "FORTRAN                = 1\n" : "";
  buffer += (ev->rh->flags.fortran_v)    ? "FORTRAN_Version        = 1\n" : "";

  // Version file paths
  if (ev->rh->c_version_path.length())
    {
    buffer += C_Version_PathTXT + "         = " + ev->rh->c_version_path +  "\n";
    }
  if (ev->rh->ada_version_path.length())
    {
    buffer += ADA_Version_PathTXT + "       = " + ev->rh->ada_version_path +  "\n";
    }
  if (ev->rh->php_version_path.length())
    {
    buffer += PHP_Version_PathTXT + "      = " + ev->rh->php_version_path +  "\n";
    }
  if (ev->rh->java_version_path.length())
    {
    buffer += JAVA_Version_PathTXT + "      = " + ev->rh->java_version_path +  "\n";
    }
  if (ev->rh->perl_version_path.length())
    {
    buffer += PERL_Version_PathTXT + "      = " + ev->rh->perl_version_path +  "\n";
    }
  if (ev->rh->fortran_version_path.length())
    {
    buffer += FORTRAN_Version_PathTXT + "   = " + ev->rh->fortran_version_path +  "\n";
    }

  // LOG
  if (ev->rh->flags.want_log)
    {
    buffer += "LOG                    = 1\n";
    if (ev->rh->flags.append_log) buffer += "APPEND_Log             = 1\n";
    else buffer += "OVERWRITE_Log          = 1\n";
    }

  if (ev->rh->flags.log_to_cerr)
    {
    buffer += "LOG_to_cerr            = 1\n";
    }

  // Versions
  buffer += "PROJECT_Want_Micro   = ";
  buffer += ((ev->rh->flags.p_want_micro) ? "1\n" : "0\n");
   
  buffer += (ev->rh->flags.p_auto_inc)   ? "PROJECT_Auto_Inc     = 1\n" : "";

  // NOTE: This should be part of individual files...
  buffer += (ev->rh->flags.f_auto_inc)   ? "FILE_Auto_Inc        = 1\n" : "";

  if (!ev->rh->version_files.empty())
    {
    buffer += Start_V_FilesTXT + "\n";
    for (i=ev->rh->version_files.begin(); i!=ev->rh->version_files.end(); ++i)
      {
      buffer += "    " + (*i) + "\n";
      }
    buffer += End_V_FilesTXT + "\n";
    }

  if (!ev->rh->no_auto_files.empty())
    {
    buffer += Start_No_Auto_FilesTXT + "\n";
    for (i=ev->rh->no_auto_files.begin(); i!=ev->rh->no_auto_files.end(); ++i)
      {
      buffer += "    " + (*i) + "\n";
      }
    buffer += End_No_Auto_FilesTXT + "\n";
    }

// Source Directories & ChangeLog 
  buffer += "\n# Source Directories and ChangeLog Path\n";
  if (!ev->rh->source_path.empty())
    {
    buffer +=  Start_SourcesTXT + "\n";
    for (i=ev->rh->source_path.begin(); i!=ev->rh->source_path.end(); ++i)
      {
      buffer += "    " + (*i) + "\n";
      }
    buffer += End_SourcesTXT + "\n";
    }

  if (!ev->rh->changelog_name.empty())
    {
    buffer += Changelog_NameTXT + " = " + ev->rh->changelog_name + "\n";
    }
  if (!ev->rh->changelog_path.empty())
    {
    buffer += Changelog_PathTXT + " = " + ev->rh->changelog_path  + "\n";
    }


  if (!ev->rh->changelog.empty())
    {
    // Write ChangeLog here as well
    fstream cl_ofp;
    string cl_name;
    string pathname = rh->changelog_path + "/" + rh->changelog_name;

    for (i=ev->rh->changelog.begin(); i!=ev->rh->changelog.end(); ++i)
      {
      if ( (*i) == "\n" )
        {
        temp += "\n";
        }
      else
        {
        temp += (*i) + "\n";
        }
      }

    tname = tf.tfile( &pathname, &cl_ofp, 3);           // Open chlogXXX for write

    if (!tname)
      {
      msg = "    FAILED to open temporary ChangeLog file for write";
      ev->rh->logfile->write2log(msg);
      }
    else
      {
      cl_name = *tname;
      msg = "    Writing \"" + cl_name + "\"";
      ev->rh->logfile->write2log(msg);


// Write temp changelog here

      cl_ofp << temp;    
      cl_ofp.close();

      // Rename it
      if (rename( (char *)cl_name.c_str(), (char *)pathname.c_str()) ) // returns zero if successful
        {
        msg = "    FAILED to rename \"" + cl_name + "\" to \"" + pathname + "\"";
        ev->rh->logfile->write2log(msg);
        }
      else
        {
        ev->rh->logfile->write2log(msg="    Wrote  \"" + pathname + "\"");
        }
      }
    
    buffer += Start_ChangelogTXT + "\n" + temp + End_ChangelogTXT + "\n\n";
    }
  buffer += "\n\n";

  if (ev->rh->write_string(ofp, buffer))
    {
    ofp.close();
    msg = "    Wrote file \"" + temp_env + "\"";
    ev->rh->logfile->report(msg);

    // Rename it
    if (rename( (char *)temp_env.c_str(), (char *)Env_FileTXT.c_str()) ) // returns zero if successful
      {
      msg = "    FAILED to rename \"" +temp_env + "\" to \"" + Env_FileTXT + "\"";
      ev->rh->logfile->write2log(msg);
      }
    else
      {
      ev->rh->logfile->write2log(msg="    Wrote  \"" + Env_FileTXT + "\"");

      // THERE IS AN ENVIRONMENT NOW, so we have an existing project
      ev->rh->flags.newproject = 0;
      }
    }
  else
    {
    ofp.close();
    msg = "    FAILED to write file \"" + temp_env + "\"";
    ev->rh->logfile->report(msg);
    }
  }
else
  {
  // ERROR
  msg = "FAILED to open temporary environment file for write";
  ev->rh->logfile->write2log(msg);
  }
}


void Environment::load_environment(void)
{
string buffer;

rh->init_flags_and_data();


// Needs to check that it is readable as well
if ( !isreadable_file(Env_FileTXT.c_str()) )   // returns TRUE if readable file exists
    {
    // PROJECT NOT LOGGED - GET DETAILS FROM USER

    // FIX UP THE LOG ===================================================================
    // Assume a log is wanted in overwite mode

    rh->flags.newproject = 1;
    rh->flags.want_log = 1;

    rh->logfile->rename_tfile();  // Rename tfile as logfile
    rh->logfile->write2log( buffer = "NOTE: Environment required for new project" );
    }

else
  {
  fstream ifp;
  string line;

  buffer = "Reading \"" + Env_FileTXT + "\"";
  rh->logfile->write2log(buffer);

  ifp.open( (char *)Env_FileTXT.c_str(), ios::in | ios::binary);
  if (ifp.is_open())
    {
    string value;

    while ( misc::getline(ifp, line, TRUE) )
      {
      while (isblank(line[0])) line = line.substr(1);

      if ( (line[0] == '#') || (line[0] == ';') || (line.length() == 0) )
        {
        continue; // comment line or empty
        }
   
// Project
      if (getvalue(line, P_NameTXT, value))
        {
        rh->p_name = value;
        }
      else if (getvalue(line, P_VersionTXT, value))
        {
        rh->get_version(value, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);
        }   
      else if (getvalue(line, P_EmailTXT, value))
        {
        rh->p_owner_email = value;
        }
      else if (getvalue(ifp, line, Start_ContactTXT, End_ContactTXT, rh->p_address))
        {
        continue;
        }
      else if (getvalue(ifp, line, Start_SummaryTXT, End_SummaryTXT, rh->p_summary))
        {
        continue;
        }

// START DATESTAMP - REQUIRED if existing project

      else if (getvalue(line, P_StartedTXT, value))
        {
        Date dt(rh);
        UNLESS (dt.date2ds((char *)value.c_str(), &rh->p_startds, &rh->p_starttz, &rh->p_start_daylight))
          {
          // ERROR
          ifp.close();
          rh->quit->HandleError(UNDERSTAND_F_ERR , Env_FileTXT.c_str(), NULL);
          return;
          }
        }

// Copyright
      else if (getvalue(line, P_OwnerTXT, value))
        {
        rh->p_owner = value;
        }      
      else if (getvalue(ifp, line, Start_CopyrightTXT, End_CopyrightTXT, rh->p_copyright))
        {
        continue;
        }

// Languages, Version Files & Log
      
// C  & CPP
      else if (getvalue(line, buffer="CPP", value))
        {
        if (value == "1") rh->flags.cpp_file = 1;
        }      
      else if (getvalue(line, buffer="C", value))
        {
        if (value == "1") rh->flags.c_file = 1;
        }      
      else if (getvalue(line, buffer="CC_Type1", value))
        {
        if (value == "1")
          {
          rh->flags.cpp_t = 1;
          rh->flags.cpp2_t = 0;
          }
        }      
      else if (getvalue(line, buffer="CC_Type2", value))
        {
        if (value == "1")
          {
          rh->flags.cpp2_t = 1;
          rh->flags.cpp_t = 0;
          }
        }      
      else if (getvalue(line, buffer="C_Version", value))
        {
        if (value == "1") rh->flags.c_v = 1;
        }      
// ADA
      else if (getvalue(line, buffer="ADA", value))
        {
        if (value == "1") rh->flags.ada_file = 1;
        }      
      else if (getvalue(line, buffer="ADA_Version", value))
        {
        if (value == "1") rh->flags.ada_v = 1;
        }      
// PHP
      else if (getvalue(line, buffer="PHP", value))
        {
        if (value == "1") rh->flags.php_file = 1;
        }      
      else if (getvalue(line, buffer="PHP_C_Type", value))
        {
        if (value == "1")
          {
          rh->flags.php_c_t = 1;
          rh->flags.php_cpp_t = 0;
          rh->flags.php_cpp2_t = 0;
          rh->flags.php_h_t = 0;
          }
        }      
      else if (getvalue(line, buffer="PHP_Type1", value))
        {
        if (value == "1")
          {
          rh->flags.php_cpp_t = 1;
          rh->flags.php_cpp2_t = 0;
          rh->flags.php_c_t = 0;
          rh->flags.php_h_t = 0;
          }

        }      
      else if (getvalue(line, buffer="PHP_Type2", value))
        {
        if (value == "1")
          {
          rh->flags.php_cpp2_t = 1;
          rh->flags.php_cpp_t = 0;
          rh->flags.php_c_t = 0;
          rh->flags.php_h_t = 0;
          }
        }      
      else if (getvalue(line, buffer="PHP_Hash_Type", value))
        {
        if (value == "1")
          {
          rh->flags.php_h_t = 1;
          rh->flags.php_cpp_t = 0;
          rh->flags.php_cpp2_t = 0;
          rh->flags.php_c_t = 0;
          }
        }      
      else if (getvalue(line, buffer="PHP_Version", value))
        {
        if (value == "1") rh->flags.php_v = 1;
        }      
// JAVA
      else if (getvalue(line, buffer="JAVA", value))
        {
        if (value == "1") rh->flags.java_file = 1;
        }      
      else if (getvalue(line, buffer="JAVA_C_Type", value))
        {
        if (value == "1")
          {
           rh->flags.java_c_t = 1;
           rh->flags.java_cpp_t = 0;
           rh->flags.java_cpp2_t = 0;
          }
        }      
      else if (getvalue(line, buffer="JAVA_CC_Type1", value))
        {
        if (value == "1")
          {
          rh->flags.java_cpp_t = 1;
          rh->flags.java_c_t = 0;
          rh->flags.java_cpp2_t = 0;
          }
        }      
      else if (getvalue(line, buffer="JAVA_CC_Type2", value))
        {
        if (value == "1")
          {
          rh->flags.java_cpp2_t = 1;
          rh->flags.java_c_t = 0;
          rh->flags.java_cpp_t = 0;
          }
        }      
      else if (getvalue(line, buffer="JAVA_Version", value))
        {
        if (value == "1") rh->flags.java_v = 1;
        }      
// PERL
      else if (getvalue(line, buffer="PERL", value))
        {
        if (value == "1") rh->flags.perl_file = 1;
        }      
      else if (getvalue(line, buffer="PERL_Version", value))
        {
        if (value == "1") rh->flags.perl_v = 1;
        }      
// FORTRAN
      else if (getvalue(line, buffer="FORTRAN", value))
        {
        if (value == "1") rh->flags.fortran_file = 1;
        }      
      else if (getvalue(line, buffer="FORTRAN_Version", value))
        {
        if (value == "1") rh->flags.fortran_v = 1;
        }

// VERSION FILE PATHS
      else if (getvalue(line, C_Version_PathTXT, value))
        {
        rh->c_version_path = value;
        rh->flags.c_v = 1;
        }
      else if (getvalue(line, ADA_Version_PathTXT, value))
        {
        rh->ada_version_path = value;
        rh->flags.ada_v = 1;
        }
      else if (getvalue(line, PHP_Version_PathTXT, value))
        {
        rh->php_version_path = value;
        rh->flags.php_v = 1;
        }
      else if (getvalue(line, JAVA_Version_PathTXT, value))
        {
        rh->java_version_path = value;
        rh->flags.java_v = 1;
        }
      else if (getvalue(line, PERL_Version_PathTXT, value))
        {
        rh->perl_version_path = value;
        rh->flags.perl_v = 1;
        }
      else if (getvalue(line, FORTRAN_Version_PathTXT, value))
        {
        rh->fortran_version_path = value;
        rh->flags.fortran_v = 1;
        }
// Miscellaneous
      else if (getvalue(line, buffer="PROJECT_Want_Micro", value))
        {
        rh->flags.p_want_micro = (value == "1") ? 1 : 0;
        }
      else if (getvalue(line, buffer="PROJECT_Auto_Inc", value))
        {
        if (value == "1") rh->flags.p_auto_inc = 1;
        }      
      else if (getvalue(line, buffer="FILE_Auto_Inc", value))  // Should be part of individual files...
        {
        if (value == "1") rh->flags.f_auto_inc = 1;
        }      
      else if (getvalue(ifp, line, Start_V_FilesTXT, End_V_FilesTXT, rh->version_files))
        {
        continue;
        }
      else if (getvalue(ifp, line, Start_No_Auto_FilesTXT, End_No_Auto_FilesTXT, rh->no_auto_files))
        {
        continue;
        }
      else if (getvalue(line, buffer="LOG", value))
        {
        if (value == "1") rh->flags.want_log = 1;
        }      
      else if (getvalue(line, buffer="APPEND_Log", value))
        {
        if (value == "1") rh->flags.append_log = 1;
        }      
      else if (getvalue(line, buffer="OVERWRITE_Log", value))
        {
        if (value == "1") rh->flags.append_log = 0;
        }      

      else if (getvalue(line, buffer="LOG_to_cerr", value))
        {
        if (value == "1") rh->flags.log_to_cerr = 1;
        }      



        
// Source Directories & ChangeLog
      else if (getvalue(ifp, line, Start_SourcesTXT, End_SourcesTXT, rh->source_path))
        {
        continue;
        }
      else if (getvalue(line, Changelog_PathTXT, value))
        {
        rh->changelog_path = value;
        }
      else if (getvalue(line, Changelog_NameTXT, value))
        {
        rh->changelog_name = value;
        }
      else if (getvalue(ifp, line, Start_ChangelogTXT, End_ChangelogTXT, rh->changelog))
        {
        continue;
        }        
      else
        {
        // UNRECOGNISED LINE
        buffer = "    Line \"" + line + "\" not recognised - SKIPPING"; 
        rh->logfile->write2log(buffer);
        }
      }

    ifp.close();

// Various checks needed here, and sort out the log file
    if ( ((rh->flags.cpp_t)||(rh->flags.cpp2_t)) && (!rh->flags.cpp_file)) // just in case...
      {
      rh->flags.cpp_t = 0;
      rh->flags.cpp2_t = 0;
      }

//    if (rh->flags.f_auto_inc && !rh->flags.f_versions) rh->flags.f_auto_inc = 0;

    if (rh->flags.want_log)
      {
      if (rh->flags.append_log)
        {
        if (rh->logfile->append_tfile())
          {
          rh->logfile->delete_tfile();
          }
        }
      else 
        { 
        // OVERWRITE MODE                
        rh->logfile->rename_tfile();  // Rename tfile as logfile, replaces existing RH.log using tfile
        }
      }
    else
      {
      // We don't want a log so delete the temp logfile and pre-existing logfile

      rh->logfile->delete_tfile();
      rh->logfile->delete_logfile();
      }

// Maybe some sort of warning if required data are  not present
// Project name, version, sources?, start datestamp, copyright owner. 


    rh->flags.environment_required = 0;
    }
  else
    {
    // ERROR - file not opened
    string msg = "    FAILED to open \"" + Env_FileTXT + "\" for read";
    rh->logfile->write2log(msg);

    }
  }

// Initial blank lines should be deleted
list<string>::iterator i;

while ( (i=rh->changelog.begin()) != rh->changelog.end() )
  {
  if ( ((*i)=="\n") || ((*i)=="") )
    {
    rh->changelog.pop_front();
    }
  else break;
  }

return;
}



void Environment::callback_yes(GtkWidget *widget, Complain *cn)
{
Environment *ev = (Environment *)(cn->data);

cn->end_complain(widget, cn);
  
// Save environment here
ev->save_environment(ev);

ev->rh->FrontPanel();
}



void Environment::env_OK_func(GtkWidget *widget, Environment *ev)
{
GtkTreeModel *model;
gboolean valid;
GtkTreeIter iter;
gchar *str;
string buffer;

//bool old_header_only = ev->rh->flags.header_only;

cout << "Environment::env_OK_func - 1" << endl;

// Project Page
ev->rh->p_name = gtk_entry_get_text(GTK_ENTRY(ev->pname_widget));
ev->rh->p_owner_email = gtk_entry_get_text(GTK_ENTRY(ev->owner_email_entry));
 
cout << "Environment::env_OK_func - 2" << endl;
 
ev->contact_el.createstringlist(&ev->contact_el, ev->rh->p_address);

cout << "Environment::env_OK_func - 3" << endl;


//delete ev->contact_el;
//ev->contact_el = NULL;


// Copyright Page
ev->rh->p_owner =  gtk_entry_get_text(GTK_ENTRY(ev->owner_widget)); 
ev->copyright_el.createstringlist(&ev->copyright_el, ev->rh->p_copyright);
//delete ev->copyright_el;
//ev->copyright_el = NULL;


 
// Languages, Version & Log
ev->rh->flags.c_file = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->c_file_widget));
ev->rh->flags.cpp_file = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->cpp_file_widget));
ev->rh->flags.ada_file = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->ada_file_widget));
ev->rh->flags.php_file = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->php_file_widget));
ev->rh->flags.java_file = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->java_file_widget));
ev->rh->flags.perl_file = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->perl_file_widget));
ev->rh->flags.fortran_file = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->fortran_file_widget));

if (ev->chosen_log_checkbox==ev->none_log_checkbox)
  {
  ev->rh->flags.want_log = 0;
  ev->rh->flags.append_log = 0;
  ev->rh->logfile->delete_logfile();  // Get rid of any log file - what about tfile?
  }
else if (ev->chosen_log_checkbox==ev->overwrite_log_checkbox)
  {
  ev->rh->flags.want_log = 1;
  ev->rh->flags.append_log = 0;
  }
else
  {
  ev->rh->flags.want_log = 1;
  ev->rh->flags.append_log = 1;  
  }
ev->rh->flags.log_to_cerr = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->log2cerr_checkbox));

ev->rh->flags.p_want_micro = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->p_want_micro_checkbutton));
ev->rh->flags.p_auto_inc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->p_auto_inc_checkbutton));
ev->rh->flags.f_auto_inc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->f_auto_inc_checkbutton));

ev->rh->flags.c_v   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->c_version_checkbox));


// cpp default is C_TYPE
ev->rh->flags.cpp_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->cpp_cpp_type_checkbox));
ev->rh->flags.cpp2_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->cpp2_cpp_type_checkbox));

ev->rh->flags.ada_v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->ada_version_checkbox));
ev->rh->flags.php_v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_version_checkbox));

ev->rh->flags.php_c_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_c_type_checkbox));
ev->rh->flags.php_cpp_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_cpp_type_checkbox));
ev->rh->flags.php_cpp2_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->php_cpp2_type_checkbox));
ev->rh->flags.php_h_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->php_hash_type_checkbox));

ev->rh->flags.java_c_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->java_c_type_checkbox));
ev->rh->flags.java_cpp_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp_type_checkbox));
ev->rh->flags.java_cpp2_t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp2_type_checkbox));

ev->rh->flags.java_v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->java_version_checkbox));
ev->rh->flags.perl_v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->perl_version_checkbox));
ev->rh->flags.fortran_v = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (ev->fortran_version_checkbox));

ev->rh->c_version_path = (ev->rh->flags.c_v) ? gtk_entry_get_text(GTK_ENTRY(ev->c_path_entry)) : "";
ev->rh->ada_version_path = (ev->rh->flags.ada_v) ? gtk_entry_get_text(GTK_ENTRY(ev->ada_path_entry)) : "";
ev->rh->php_version_path = (ev->rh->flags.php_v) ? gtk_entry_get_text(GTK_ENTRY(ev->php_path_entry)) : "";
ev->rh->java_version_path = (ev->rh->flags.java_v) ? gtk_entry_get_text(GTK_ENTRY(ev->java_path_entry)) : "";
ev->rh->perl_version_path = (ev->rh->flags.perl_v) ? gtk_entry_get_text(GTK_ENTRY(ev->perl_path_entry)) : "";
ev->rh->fortran_version_path = (ev->rh->flags.fortran_v) ? gtk_entry_get_text(GTK_ENTRY(ev->fortran_path_entry)) : "";




// If we have changed directories to an existing project
// we should detect it and change flag

ev->rh->flags.environment_required = 0; // We have a environment

// Source Directories & ChangeLog
ev->rh->source_path.clear();
cout << "Environment::env_OK_func - source list 1:-" << endl;
{
list<string>::iterator s;  
for (s=ev->rh->source_path.begin(); s!=ev->rh->source_path.end(); ++s)
  {
  cout << "    " << *s << endl;
  }
}
model = gtk_tree_view_get_model(GTK_TREE_VIEW(ev->treeview));
valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
while (valid)
  {
  gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                    0, // the column #,
                    &str,
                    -1);
  ev->rh->source_path.push_back(str);

  g_free(str);
  valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
  }
  
{
cout << "Environment::env_OK_func - source list 2:-" << endl;
list<string>::iterator s;  
for (s=ev->rh->source_path.begin(); s!=ev->rh->source_path.end(); ++s)
  {
  cout << "    " << *s << endl;
  }
}
// Get ChangeLog path  
if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->changelog_checkbox))==TRUE)
  {
  ev->rh->changelog_path = gtk_entry_get_text(GTK_ENTRY(ev->changelog_path_entry));
  }
else ev->rh->changelog_path.clear();

// Get ChangeLog name 
ev->rh->changelog_name = gtk_entry_get_text(GTK_ENTRY(ev->changelog_name_entry));

// ========================================================================  
// Introduce a check that ChangeLog and source paths are within the project
// ========================================================================  
list<string>::iterator src;
buffer.clear();
int num_warns = 0;  
{
cout << "Environment::env_OK_func - source list 3:-" << endl;
list<string>::iterator s;  
for (s=ev->rh->source_path.begin(); s!=ev->rh->source_path.end(); ++s)
  {
  cout << "    " << *s << endl;
  }
}
if ( ( (ev->rh->changelog_path.length() >= ev->rh->project_path.length()) &&
  (ev->rh->changelog_path.substr(0, ev->rh->project_path.length()) != ev->rh->project_path) ) ||
  ((!ev->rh->changelog_path.empty()) && (ev->rh->changelog_path.length() < ev->rh->project_path.length())) )    
  {
  num_warns++;
  buffer += "\nChangeLog path \"" +  ev->rh->changelog_path + "\"";
  }

for (src = ev->rh->source_path.begin();
          src!=ev->rh->source_path.end();
          ++src)
  {
  if ( ( (*src).length() >= ev->rh->project_path.length() ) &&
    ( (*src).substr(0, ev->rh->project_path.length()) != ev->rh->project_path ) ||
      ( (*src).length() < ev->rh->project_path.length() ) )
    {
    num_warns++;
    buffer += "\nSource path \"" +  (*src) + "\"";
    }
  }

{
cout << "Environment::env_OK_func - source list 4:-" << endl;
list<string>::iterator s;  
for (s=ev->rh->source_path.begin(); s!=ev->rh->source_path.end(); ++s)
  {
  cout << "    " << *s << endl;
  }
}

if (num_warns)
  {
  Complain *cn = new Complain(ev->rh->icon_pixbuf, &lightgrey, &pink, GTK_STOCK_CANCEL, GTK_STOCK_OK);

  string warning =   "    WARNING!\nThe following ";
  warning += ((num_warns > 1) ? "are" : "is");
  warning += " not in the project path\nwhich is \"" +
             ev->rh->project_path + "\".\n" + buffer + "\nIs this OK?"; 
    
  cn->complain(CN_YES_NO,
                (char *)"PATH WARNING",
                (char *)warning.c_str(),
                callback_yes, NULL, ev);
  return;
  }

{
cout << "Environment::env_OK_func - source list 5:-" << endl;
list<string>::iterator s;  
for (s=ev->rh->source_path.begin(); s!=ev->rh->source_path.end(); ++s)
  {
  cout << "    " << *s << endl;
  }
}  
// Save environment here
ev->save_environment(ev);

if (!ev->existing_project_flag)
  {
  ev->load_environment(); // Make sure it is OK
  }
{
cout << "Environment::env_OK_func - source list 6:-" << endl;
list<string>::iterator s;  
for (s=ev->rh->source_path.begin(); s!=ev->rh->source_path.end(); ++s)
  {
  cout << "    " << *s << endl;
  }
}
ev->rh->FrontPanel();
}

void Environment::callback_write_RH_run(GtkWidget *widget, Environment *ev)
{
string pathname;
string buffer;
ofstream ofp;
  
//if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (shell_widget)))
pathname = ev->rh->project_path + "/" + RH_runTXT;
ofp.open( pathname.c_str(), ios::out | ios::binary); 

UNLESS (ofp.is_open())
  {
  buffer = "Can't open file \"" + pathname + "\" for write";
  ev->rh->logfile->report(pathname);
  }
else
  {
  buffer = "#!/bin/sh\n\ncd " + ev->rh->project_path + "\n";
  buffer += ev->rh->progname + " &\nexit 0\n\n";
        
  ofp.write( (char *)buffer.c_str(), buffer.length() );
  ofp.close();
  chmod(pathname.c_str(), 0775);
  buffer = "Created shell executable \"" + pathname + "\"";

  cout << "Environment::write_RH_run - " << buffer << endl;

  ev->rh->logfile->report(buffer);
  }
}


// We quit completely
void Environment::abort_func(GtkWidget *widget, Environment *ev)
{
cout << "Environment::abort_func" << endl;

ev->rh->DestroyWidgets();
ev->rh->flags.abort = 1;
ev->rh->quit->HandleError(USER_ABORT_ERR, NULL, NULL);
}


// We quit keeping the existing values in RevHead
void Environment::cancel_func(GtkWidget *widget, Environment *ev)
{
cout << "Environment::cancel_func" << endl;
ev->rh->FrontPanel();
}


void Environment::add_columns (GtkTreeView *treeview, gchar *title)
{
GtkCellRenderer *renderer;
GtkTreeViewColumn *column;

renderer = gtk_cell_renderer_text_new ();
column = gtk_tree_view_column_new_with_attributes(title,
                 renderer,
                 "text",
                 0,
                 NULL);
gtk_tree_view_column_set_sort_column_id (column, 0);
gtk_tree_view_append_column(treeview, column);

}





GtkTreeModel *Environment::create_model(list<string> mlist)
{
GtkListStore *store;
GtkTreeIter iter;
list<string>::iterator i;

store = gtk_list_store_new (1, G_TYPE_STRING);

if (mlist.empty())
  {
  // Add the project directory
  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, (gchar *)rh->project_path.c_str(), -1);  
  }
else for (i=mlist.begin(); i!=mlist.end(); ++i)
  {
  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, (gchar *)i->c_str(), -1);
  }

return GTK_TREE_MODEL(store);
}


GtkWidget *Environment::get_list_store(list<string> mlist)
{
GtkWidget *sw;
GtkTreeModel *model;


sw = gtk_scrolled_window_new (NULL, NULL);
gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
             GTK_SHADOW_ETCHED_IN);
gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
              GTK_POLICY_AUTOMATIC,
              GTK_POLICY_AUTOMATIC);

model = create_model(mlist);

treeview = gtk_tree_view_new_with_model(model);
gtk_tree_view_set_rules_hint(GTK_TREE_VIEW (treeview), TRUE);
gtk_tree_view_set_search_column(GTK_TREE_VIEW (treeview),
               0);

g_object_unref (model);

gtk_container_add (GTK_CONTAINER (sw), treeview);

/* add columns to the tree view */
add_columns(GTK_TREE_VIEW (treeview), (gchar *)"Source Directories");

gtk_widget_show_all(sw);
return sw;
}



void Environment::add_new_source(string pathname, Environment *ev)
{
GtkTreeModel* model;
gboolean valid;
GtkTreeIter iter;
GtkTreeIter sibling;
gchar *str;
string name;

if (!pathname.length()) return;

model = gtk_tree_view_get_model(GTK_TREE_VIEW(ev->treeview));

valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

while (valid)
  {
  gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                    0, // the column #,
                    &str,
                    -1);
  name = str;
  g_free(str);

  if (pathname == name)
    {
    return;  // Don't add  if already present
    }
  else if (strcasecmp(pathname.c_str(), name.c_str()) < 0)
    {
    // INSERT IT HERE
    gtk_list_store_insert_before(GTK_LIST_STORE(model),
               &sibling,
               &iter); 
    gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0, pathname.c_str(), -1);
    ev->rh->source_path.push_back(pathname);
    break;
    }
  
  valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
  }

if (!valid)
  {
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set (GTK_LIST_STORE(model), &iter, 0, pathname.c_str(), -1);
  }
}


void Environment::callback_delete_source(GtkWidget *widget, Environment *ev)
{
GtkTreeIter iter;
GtkTreeSelection *select;
GtkTreeModel *model;

model = gtk_tree_view_get_model(GTK_TREE_VIEW(ev->treeview));
select = gtk_tree_view_get_selection (GTK_TREE_VIEW(ev->treeview));
gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

if ( (gtk_tree_selection_get_selected(select, &model, &iter)) )
  {
  gchar *str;
  gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
           0, // the column #,
           &str,
           -1);    

  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
  g_free(str);
  }
}


void Environment::callback_add_source(GtkWidget *widget, Environment *ev)
{
gchar *path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(ev->directory_chooser));

if (path && *path) ev->add_new_source(path, ev);
g_free(path);
}


void Environment::callback_assign_changelog(GtkWidget *widget, Environment *ev)
{
gchar *path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(ev->directory_chooser));
if (path && *path) gtk_entry_set_text(GTK_ENTRY(ev->changelog_path_entry), path);
g_free(path);

}


void Environment::callback_want_changelog(GtkWidget *widget, Environment *ev)
{
if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))==TRUE)
  {
  cout << "Environment::callback_want_changelog - SET" << endl;
  gtk_widget_set_sensitive(GTK_WIDGET(ev->changelog_path_entry), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(ev->changelog_button), TRUE);
  }
else
  {
  cout << "Environment::callback_want_changelog - NOT SET" << endl;
  gtk_widget_set_sensitive(GTK_WIDGET(ev->changelog_path_entry), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(ev->changelog_button), FALSE);  
  }
}


void Environment::c_file_widget_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->c_c_type_checkbox && ev->c_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->c_file_widget)))
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->c_c_type_checkbox), TRUE);
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->c_c_type_checkbox), TRUE);

		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->c_c_type_checkbox), TRUE);
		}
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->c_c_type_checkbox), FALSE);
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->c_c_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->c_c_type_checkbox), FALSE);
		}
  }
}

void Environment::c_c_type_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->c_c_type_checkbox && ev->c_file_widget)
  {
  if ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->c_c_type_checkbox))) &&
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->c_file_widget)) )
    {
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->c_c_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->c_c_type_checkbox), TRUE);
		}
  }
}

void Environment::cpp_file_widget_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->cpp_c_type_checkbox && ev->cpp_cpp_type_checkbox && ev->cpp2_cpp_type_checkbox && ev->cpp_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->cpp_file_widget))==TRUE)
    {
    if  ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->cpp_c_type_checkbox))) &&
          (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->cpp_cpp_type_checkbox))) )
      {
      // C_TYPE is the default
      gtk_widget_set_sensitive(GTK_WIDGET(ev->cpp_cpp_type_checkbox), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(ev->cpp2_cpp_type_checkbox), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(ev->cpp_c_type_checkbox), TRUE);      

//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), TRUE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), TRUE);
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE);
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);
      }
    }
  else
    {
    // Turn them all off
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), FALSE);
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE);
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);

    gtk_widget_set_sensitive(GTK_WIDGET(ev->cpp_cpp_type_checkbox), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(ev->cpp_c_type_checkbox), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(ev->cpp2_cpp_type_checkbox), FALSE);
    }
  }
}

void Environment::cpp_type_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->cpp_c_type_checkbox && ev->cpp_cpp_type_checkbox && ev->cpp2_cpp_type_checkbox && ev->cpp_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))==TRUE)
    {
    if (widget == ev->cpp_c_type_checkbox)
      {
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE);
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);

			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);
      }
    else if (widget == ev->cpp_cpp_type_checkbox)
      {
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), FALSE);
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);

			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp2_cpp_type_checkbox), FALSE);
      }
    else if (widget == ev->cpp2_cpp_type_checkbox)
      {
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), FALSE);
//      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE); 

			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_c_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->cpp_cpp_type_checkbox), FALSE);				
      }
    }
  }
}







/**
void Environment::c_version_checkbox_clicked(GtkWidget *widget, Environment *ev)
{  
if (ev->c_version_checkbox && ev->cpp_version_checkbox &&
    ev->c_file_widget && ev->cpp_file_widget)
  {
  if ( (widget==ev->c_version_checkbox) || (widget==ev->cpp_version_checkbox) )
    {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))==TRUE)
      {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_version_checkbox), TRUE);
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->c_version_checkbox), TRUE);
      }
    else
      {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->cpp_version_checkbox), FALSE);
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->c_version_checkbox), FALSE);
      }
    }
  }
}
**/

void Environment::ada_file_widget_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->ada_type_checkbox && ev->ada_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->ada_file_widget)))
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->ada_type_checkbox), TRUE);
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->ada_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->ada_type_checkbox), TRUE);
		}
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->ada_type_checkbox), FALSE);
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->ada_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->ada_type_checkbox), FALSE);
		}
  }
}

void Environment::ada_type_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->ada_type_checkbox && ev->ada_file_widget)
  {
  if ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->ada_type_checkbox))) &&
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->ada_file_widget)) )
    {
//    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ev->ada_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->ada_type_checkbox), TRUE);
		}
  }
}

void Environment::php_file_widget_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->php_c_type_checkbox &&  ev->php_cpp_type_checkbox &&  ev->php_cpp_type_checkbox &&
    ev->php_hash_type_checkbox && ev->php_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_file_widget)))
    {
    if (  (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_c_type_checkbox))) &&
          (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_cpp_type_checkbox))) &&
          (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_cpp2_type_checkbox))) &&
          (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_hash_type_checkbox))) )
      {
      gtk_widget_set_sensitive(GTK_WIDGET(ev->php_c_type_checkbox), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(ev->php_cpp_type_checkbox), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(ev->php_cpp2_type_checkbox), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(ev->php_hash_type_checkbox), TRUE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_c_type_checkbox), TRUE);
			}
    }
  else
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_c_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp2_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_hash_type_checkbox), FALSE);


			
    gtk_widget_set_sensitive(GTK_WIDGET(ev->php_c_type_checkbox), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(ev->php_cpp_type_checkbox), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(ev->php_cpp2_type_checkbox), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(ev->php_hash_type_checkbox), FALSE);
    
    }
  }
}

void Environment::php_type_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->php_c_type_checkbox && ev->php_cpp_type_checkbox  && ev->php_cpp2_type_checkbox&&
    ev->php_hash_type_checkbox && ev->php_file_widget)
  {
  if ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_c_type_checkbox))) &&
      (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_cpp_type_checkbox))) &&
      (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_cpp2_type_checkbox))) &&
      (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_hash_type_checkbox))) &&
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_file_widget)) )
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_c_type_checkbox), TRUE);
		}
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_file_widget)))
    {
    if ((widget==ev->php_c_type_checkbox) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_c_type_checkbox)))
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp2_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_hash_type_checkbox), FALSE);

			}
    else if ((widget==ev->php_cpp_type_checkbox) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_cpp_type_checkbox)))
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_c_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp2_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_hash_type_checkbox), FALSE);

			}
    else if ((widget==ev->php_cpp2_type_checkbox) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_cpp2_type_checkbox)))
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_c_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_hash_type_checkbox), FALSE);
			}
    else if ((widget==ev->php_hash_type_checkbox) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->php_hash_type_checkbox)))
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_c_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->php_cpp2_type_checkbox), FALSE);
			}
    }
  }
}

void Environment::java_file_widget_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->java_c_type_checkbox && ev->java_cpp_type_checkbox && ev->java_cpp2_type_checkbox && ev->java_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_file_widget)))
    {
    if ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_c_type_checkbox))) &&
        (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp2_type_checkbox))) &&
        (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp_type_checkbox))) )
      {
      gtk_widget_set_sensitive(GTK_WIDGET(ev->java_c_type_checkbox), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(ev->java_cpp_type_checkbox), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(ev->java_cpp2_type_checkbox), TRUE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_c_type_checkbox), TRUE);
			}
    }
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->java_c_type_checkbox), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(ev->java_cpp_type_checkbox), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(ev->java_cpp2_type_checkbox), FALSE);

		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_c_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_cpp_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_cpp2_type_checkbox), FALSE);
		}
  }
}

void Environment::java_type_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->java_c_type_checkbox && ev->java_cpp_type_checkbox && ev->java_cpp2_type_checkbox && ev->java_file_widget)
  {
  if ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_c_type_checkbox))) &&
      (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp_type_checkbox))) &&
      (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp2_type_checkbox))) &&
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_file_widget)) )
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_c_type_checkbox), TRUE);
    }
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_file_widget)))
    {
    if ((widget==ev->java_c_type_checkbox) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_c_type_checkbox)))
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_cpp_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_cpp2_type_checkbox), FALSE);
			}
    else if ((widget==ev->java_cpp_type_checkbox) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp_type_checkbox)))
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_c_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_cpp2_type_checkbox), FALSE);
			}
    else if ((widget==ev->java_cpp2_type_checkbox) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->java_cpp2_type_checkbox)))
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_c_type_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->java_cpp_type_checkbox), FALSE);
			}
    }
  }
}


void Environment::log_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->none_log_checkbox && ev->append_log_checkbox && ev->overwrite_log_checkbox)
  {
  if (widget == ev->none_log_checkbox)
    {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->none_log_checkbox))==TRUE)
      {
      ev->chosen_log_checkbox = ev->none_log_checkbox;

			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->append_log_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->overwrite_log_checkbox), FALSE);	
      }
    else if (ev->chosen_log_checkbox == ev->none_log_checkbox)
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->none_log_checkbox), TRUE);
			}
    }
  else if (widget == ev->append_log_checkbox)
    {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->append_log_checkbox))==TRUE)
      {
      ev->chosen_log_checkbox = ev->append_log_checkbox;

			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->none_log_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->overwrite_log_checkbox), FALSE);
			}
    else if (ev->chosen_log_checkbox == ev->append_log_checkbox)
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->none_log_checkbox), TRUE);
			}    
    }
  else if (widget == ev->overwrite_log_checkbox)
    {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->overwrite_log_checkbox))==TRUE)
      {
      ev->chosen_log_checkbox = ev->overwrite_log_checkbox;
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->none_log_checkbox), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->append_log_checkbox), FALSE);
			}
    else if (ev->chosen_log_checkbox == ev->overwrite_log_checkbox)
      {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ev->none_log_checkbox), TRUE);
			}    
    }
  }
}

void Environment::perl_file_widget_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->perl_hash_type_checkbox && ev->perl_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->perl_file_widget)))
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->perl_hash_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->perl_hash_type_checkbox), TRUE);
		}
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->perl_hash_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->perl_hash_type_checkbox), FALSE);
		}
  }
}


void Environment::perl_hash_type_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->perl_file_widget && ev->perl_hash_type_checkbox)
  {
  if ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->perl_hash_type_checkbox))) &&
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->perl_file_widget)) )
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->perl_hash_type_checkbox), TRUE);
		}
  }
}

void Environment::fortran_file_widget_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->fortran_type_checkbox && ev->fortran_file_widget)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->fortran_file_widget)))
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->fortran_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->fortran_type_checkbox), TRUE);
		}
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ev->fortran_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->fortran_type_checkbox), FALSE);
		}
  }
}

void Environment::fortran_type_checkbox_clicked(GtkWidget *widget, Environment *ev)
{
if (ev->fortran_type_checkbox && ev->fortran_file_widget)
  {
  if ( (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->fortran_type_checkbox))) &&
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->fortran_file_widget)) )
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ev->fortran_type_checkbox), TRUE);
		}
  }
}


string Environment::get_browse_directory(string title, Environment *ev)
{
GtkWidget *box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0); // The new container
gtk_widget_reparent(GTK_WIDGET(main_grid), box);

ChangeDir *cd = new ChangeDir(ev->rh); // Get dirname only
	
cd->change_dir(title, TRUE);
gtk_main();

// NOTE: When cd->change_dir exits,
//	     both "Cancel" and "OK" MUST be made to come here.
//       It has to perform a gtk_main_quit() from both "Cancel" and "OK".

string path = cd->get_dirname();

cout << "Environment::get_browse_directory - path= " << path << endl;
	


delete cd;
	
ev->rh->DestroyWidgets();
gtk_widget_reparent(GTK_WIDGET(main_grid), ev->rh->vbox);
gtk_widget_show_all (GTK_WIDGET(ev->rh->vbox));	

	
return path;
}






// Callbacks for version directories
// C/C++
void Environment::callback_browse_c(GtkWidget *widget, Environment *ev)
{
cout << "Environment::callback_browse_c" << endl;

string path = ev->get_browse_directory("Directory (C/C++ Version File)", ev);

cout << "Environment::callback_browse_c - path= " << path << endl;

	
if (!path.empty())
  {
  gtk_entry_set_text(GTK_ENTRY(ev->c_path_entry), path.c_str());
  }
}




void Environment::callback_delete_c(GtkWidget *widget, Environment *ev)
{
//cout << "Environment::callback_delete_c" << endl;
gtk_entry_set_text(GTK_ENTRY(ev->c_path_entry), "");
}

// ADA
void Environment::callback_browse_ada(GtkWidget *widget, Environment *ev)
{
cout << "Environment::callback_browse_ada" << endl;

string path = ev->get_browse_directory("Directory (ADA Version File)", ev);
//cout << "Environment::callback_browse_ada - path= " << path << endl;
if (!path.empty())
  {
  gtk_entry_set_text(GTK_ENTRY(ev->ada_path_entry), path.c_str());
  }
}
void Environment::callback_delete_ada(GtkWidget *widget, Environment *ev)
{
//cout << "Environment::callback_delete_ada" << endl;
gtk_entry_set_text(GTK_ENTRY(ev->ada_path_entry), "");
}

// PHP
void Environment::callback_browse_php(GtkWidget *widget, Environment *ev)
{
cout << "Environment::callback_browse_php" << endl;
string path = ev->get_browse_directory("Directory (PHP Version File)", ev);
if (!path.empty())
  {
  gtk_entry_set_text(GTK_ENTRY(ev->php_path_entry), path.c_str());
  }
}
void Environment::callback_delete_php(GtkWidget *widget, Environment *ev)
{
//cout << "Environment::callback_delete_ada" << endl;
gtk_entry_set_text(GTK_ENTRY(ev->php_path_entry), "");
}

// JAVA
void Environment::callback_browse_java(GtkWidget *widget, Environment *ev)
{
cout << "Environment::callback_browse_java" << endl;
	
string path = ev->get_browse_directory("Directory (JAVA Version File)", ev);
if (!path.empty())
  {
  gtk_entry_set_text(GTK_ENTRY(ev->java_path_entry), path.c_str());
  }
}
void Environment::callback_delete_java(GtkWidget *widget, Environment *ev)
{
//cout << "Environment::callback_delete_java" << endl;
gtk_entry_set_text(GTK_ENTRY(ev->java_path_entry), "");
}

// PERL
void Environment::callback_browse_perl(GtkWidget *widget, Environment *ev)
{
cout << "Environment::callback_browse_perl" << endl;
	
string path = ev->get_browse_directory("Directory (PERL Version File)", ev);
if (!path.empty())
  {
  gtk_entry_set_text(GTK_ENTRY(ev->perl_path_entry), path.c_str());
  }
}
void Environment::callback_delete_perl(GtkWidget *widget, Environment *ev)
{
//cout << "Environment::callback_delete_perl" << endl;
gtk_entry_set_text(GTK_ENTRY(ev->perl_path_entry), "");
}

// FORTRAN
void Environment::callback_browse_fortran(GtkWidget *widget, Environment *ev)
{
cout << "Environment::callback_browse_fortran" << endl;
	
string path = ev->get_browse_directory("Directory (FORTRAN Version File)", ev);
if (!path.empty())
  {
  gtk_entry_set_text(GTK_ENTRY(ev->fortran_path_entry), path.c_str());
  }
}
void Environment::callback_delete_fortran(GtkWidget *widget, Environment *ev)
{
//cout << "Environment::callback_delete_fortran" << endl;
gtk_entry_set_text(GTK_ENTRY(ev->fortran_path_entry), "");
}




//============================================================================
//                              PROJECT PAGE
//============================================================================
void Environment::fill_project_page(GtkWidget *vbox)
{
GtkWidget *wid, *grid, *frame, *box;
string buffer;

box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);


// PROJECT FRAME --------------------------------------------------
frame = gtk_frame_new("Project:");
		
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);


grid = gtk_grid_new();
gtk_grid_set_row_homogeneous (GTK_GRID(grid), FALSE);
gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);

		
gtk_container_set_border_width(GTK_CONTAINER(grid), 10);
	
gtk_container_add(GTK_CONTAINER(frame), grid);

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Project\n"
      "Project Name:\n  The name for this project without any version number\n"
      "Project's Email:\n  The email address for the whole project");
	
// PPROJECT NAME
wid = gtk_label_new( RevHead::P_NameTXT.c_str() );

// --- Align the label to the right side.
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);


gtk_grid_attach (GTK_GRID(grid), wid, 0, 1, 20, 8);
	
// This will need to be made longer
pname_widget = gtk_entry_new();

gtk_entry_set_max_length(	GTK_ENTRY(pname_widget), 30);

if ( existing_project_flag && (!rh->p_name.empty()) )		// EXISTING PROJECT
	{
	gtk_entry_set_text(GTK_ENTRY(pname_widget),   rh->p_name.c_str() );
	}
else
	{
   gtk_entry_set_text(GTK_ENTRY(pname_widget), "(UNNAMED PROJECT)" );
	}

	
gtk_grid_attach (GTK_GRID(grid), pname_widget, 20, 1, 30, 8);


// Project Email Address
wid = gtk_label_new( RevHead::P_EmailTXT.c_str());
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);
	
gtk_grid_attach (GTK_GRID(grid), wid, 0, 9, 20, 8);

owner_email_entry = gtk_entry_new();

gtk_entry_set_max_length(	GTK_ENTRY(owner_email_entry), 40);
if (existing_project_flag)
	{
	gtk_entry_set_text(GTK_ENTRY(owner_email_entry), (char *)rh->p_owner_email.c_str() );
	}
else gtk_entry_set_text(GTK_ENTRY(owner_email_entry), rh->np_owner_email.c_str() );

gtk_grid_attach (GTK_GRID(grid), owner_email_entry, 20, 9, 30, 8);

// CONTACT FRAME --------------------------------------------------

frame = gtk_frame_new(RevHead::ContactTXT.c_str());
	
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 5);

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Contact\n"
      "Contact information\n"
      "  Maybe email and snail addresses and URL");

                            
wid = contact_el.create_entry_list();
		
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);
	
gtk_container_add(GTK_CONTAINER(frame), wid);
	

if ( (existing_project_flag) && (!rh->p_address.empty()) ) // i.e. existing not empty
  {
  list<string>::iterator i;

  for (i=rh->p_address.begin(); i!=rh->p_address.end(); ++i)
    {
    contact_el.add_to_elist( (*i) , &contact_el);
    }
  }
else if (!rh->np_address.empty())         // i.e. defaults not empty
  {
  list<string>::iterator i;

  for (i=rh->np_address.begin(); i!=rh->np_address.end(); ++i)
    {
    contact_el.add_to_elist( (*i) , &contact_el);
    }
  }
else
    {
    for (int n=3; n; n--)                 // we want 3 entries anyway
        {
        contact_el.add_to_elist("", &contact_el);
        }
    }

}


// ==============
// COPYRIGHT PAGE
// ==============
void Environment::fill_copyright_page(GtkWidget *vbox)
{
GtkWidget *wid, *grid, *frame, *box;


gtk_widget_set_tooltip_text (GTK_WIDGET(vbox),
      "Copyright\n"
      "Enter copyright information here");


// Project Owner ======================================================

box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);

gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 5);
	
grid = gtk_grid_new();

gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
	
gtk_widget_show (grid);

gtk_box_pack_start(GTK_BOX(box), grid, TRUE, TRUE, 5);


// COPYRIGHT OWNER
wid = gtk_label_new ( (char *)RevHead::P_OwnerTXT.c_str() );
gtk_misc_set_alignment (GTK_MISC (wid), 1, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 4, 1, 16, 5); // L, T, W, H



owner_widget = gtk_entry_new();
gtk_entry_set_max_length(	GTK_ENTRY(owner_widget), 40);
if (existing_project_flag && (!rh->p_owner.empty()))
	{
  gtk_entry_set_text(GTK_ENTRY(owner_widget), (char *)rh->p_owner.c_str() );
	}
else gtk_entry_set_text(GTK_ENTRY(owner_widget), (char *)rh->np_owner.c_str() );
  

gtk_grid_attach (GTK_GRID(grid), owner_widget, 20, 1, 40, 5); // L, T, W, H

gtk_widget_set_tooltip_text (GTK_WIDGET(owner_widget),
       "Copyright Owner\n"
      "Enter copyright owner here");

 
  
// PROJECT COPYRIGHT

box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_set_homogeneous (GTK_BOX (box), TRUE);
gtk_grid_attach (GTK_GRID(grid), box, 2, 60, 60, 40); // L, T, W, H
		
frame = gtk_frame_new(RevHead::P_CopyrightTXT.c_str());

gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 5);


gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Copyright Notice\n"
      "Enter copyright information here");

wid = copyright_el.create_entry_list();

gtk_container_add(GTK_CONTAINER(frame), wid);
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);

if ( (existing_project_flag) && (!rh->p_copyright.empty()) ) // i.e. existing not empty
  {
  list<string>::iterator i;
  for (i=rh->p_copyright.begin(); i!=rh->p_copyright.end(); ++i)
    {
    copyright_el.add_to_elist( (*i) , &copyright_el);
    }
  }
else if (!rh->np_copyright.empty())         // i.e. defaults not empty
  {
  list<string>::iterator i;
  for (i=rh->np_copyright.begin(); i!=rh->np_copyright.end(); ++i)
    {
    copyright_el.add_to_elist( (*i) , &copyright_el);
    }
  } 
else for (int n=3; n; n--)                  // we want 3 entries anyway
  {
  copyright_el.add_to_elist("", &copyright_el);
  }

}




// =============================================================================
//                            DIRECTORIES PAGE
// =============================================================================
void Environment::callback_directory_selection(GtkWidget *widget, Environment *ev)
{
cout << "Environment::callback_directory_selection" << endl;
	
ev->dir_pathname.clear();
	
char *dirname  = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(ev->directory_chooser));

if (dirname && *dirname)
	{
	cout <<"  Directory: " << dirname << endl;
	ev->dir_pathname = dirname;
	}
else 
	{
	cout <<"  Directory: - nothing returned!" << endl;
	}

}

void Environment::fill_directories_page(GtkWidget *vbox)
{
GtkWidget *frame, *wid, *grid;

	
gtk_widget_set_tooltip_text (GTK_WIDGET(vbox),
      "Select Directory\n"
      "  Double Click to move to a directory.\n"
      "  Single Click to select that name.\n"
      "  A selected directory appears in \"NAME\".\n"
      "A selected directory can be added to the source\n"
      "  directory list or, if \"Want ChangeLog Directory\"\n"
      "  is selected, click on the ChangeLog button to choose\n"
      "  a directory for a separate ChangLog.\n"
      "To remove a source directory select it in the source\n"
      "  directory list and click \"Delete Selected Directory\".");
	

grid = gtk_grid_new ();

gtk_box_pack_start (GTK_BOX(vbox), grid, TRUE, TRUE, 0);

gtk_grid_set_row_homogeneous (GTK_GRID(grid), TRUE);
gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
	
frame = gtk_frame_new("Select Directory");
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_grid_attach (GTK_GRID(grid), frame, 0, 0, 64, 45);
	
	
directory_chooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
gtk_file_chooser_set_create_folders (GTK_FILE_CHOOSER(directory_chooser), FALSE);

g_signal_connect(G_OBJECT(directory_chooser), "selection-changed", G_CALLBACK(callback_directory_selection), this);
	
gtk_container_add(GTK_CONTAINER(frame), directory_chooser);



changelog_button = gtk_button_new_with_label("ChangeLog Path");
decorate_widget(changelog_button, &black, NULL, NULL, NULL, NULL);

g_signal_connect (G_OBJECT (changelog_button), "clicked",
            G_CALLBACK(callback_assign_changelog), this);
                                 
gtk_grid_attach (GTK_GRID(grid), changelog_button, 1, 45, 15, 6); // L, T, W, H

  
wid = gtk_button_new_with_label("Add Source Directory");

decorate_widget(wid, &black, NULL, NULL, NULL, NULL);

g_signal_connect (G_OBJECT (wid), "clicked",
            G_CALLBACK(callback_add_source), this);                    

gtk_grid_attach (GTK_GRID(grid), wid, 17, 45, 17, 6); // L, T, W, H


	
wid = get_list_store(rh->source_path);
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);
gtk_grid_attach (GTK_GRID(grid), wid, 34, 45, 30, 23); // L, T, W, H

  
wid = gtk_button_new_with_label("Delete Selected Directory");
decorate_widget(wid, &black, NULL, NULL, NULL, NULL);
gtk_container_set_border_width(GTK_CONTAINER(wid), 10);

g_signal_connect (G_OBJECT (wid), "clicked",
            G_CALLBACK(callback_delete_source), this);

gtk_grid_attach (GTK_GRID(grid), wid, 40, 68, 20, 6); // L, T, W, H

	
frame = gtk_frame_new("ChangeLog");
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_grid_attach (GTK_GRID(grid), frame, 0, 51, 35, 21); // L, T, W, H

	

changelog_checkbox = gtk_check_button_new_with_label("Want ChangeLog Directory");
g_signal_connect (G_OBJECT (changelog_checkbox), "clicked",
            G_CALLBACK(callback_want_changelog), this);                    
gtk_grid_attach (GTK_GRID(grid), changelog_checkbox, 4, 55, 20, 5); // L, T, W, H


wid = gtk_label_new("Path: ");
gtk_misc_set_alignment (GTK_MISC (wid), 1, 0.5);

gtk_grid_attach (GTK_GRID(grid), wid, 0, 60, 5, 5); // L, T, W, H


changelog_path_entry = gtk_entry_new();
gtk_editable_set_editable(GTK_EDITABLE(changelog_path_entry), FALSE);

gtk_grid_attach (GTK_GRID(grid), changelog_path_entry, 5, 60, 28, 5); // L, T, W, H


if (rh->changelog_path.empty())
  {
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(changelog_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(changelog_path_entry), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(changelog_button), FALSE);    
  }
else
  {
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(changelog_checkbox), TRUE);
  gtk_entry_set_text(GTK_ENTRY(changelog_path_entry), rh->changelog_path.c_str());
  gtk_widget_set_sensitive(GTK_WIDGET(changelog_path_entry), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(changelog_button), TRUE);
  }

wid = gtk_label_new("Name:");
gtk_misc_set_alignment (GTK_MISC (wid), 1, 0.5);

gtk_grid_attach (GTK_GRID(grid), wid, 0, 65, 5, 5); // L, T, W, H


changelog_name_entry = gtk_entry_new();
gtk_entry_set_text(GTK_ENTRY(changelog_name_entry), rh->changelog_name.c_str());
gtk_grid_attach (GTK_GRID(grid), changelog_name_entry, 5, 65, 28, 5); // L, T, W, H




}

// ==============
// LANGUAGES PAGE
// ==============
void Environment::fill_language_page(GtkWidget *vbox)
{
GtkWidget *wid, *grid, *frame, *box;

// LANGUAGE INFO
frame = gtk_frame_new("Language Preferences");
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "Language Preferences\n"
      "Language\n"
      "  Choose which language(s) is/are required (or none).\n"
      "  Whwn a language is chosen its default comment type is ticked.\n"
      "  Some languages allow alternative comment types.\n");

gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);


grid = gtk_grid_new();
gtk_container_set_border_width(GTK_CONTAINER(grid), 20);


gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
gtk_grid_set_row_spacing (GTK_GRID(grid), 5);
gtk_container_add(GTK_CONTAINER(frame), grid);  

                             
  
wid = gtk_label_new("Language");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 0, 0, 16, 5); // L, T, W, H
                   

                   
wid = gtk_label_new("Comment  Type");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 30, 0, 34, 5); // L, T, W, H


                   
// ------  C horizontally ------------------------------------------------------
c_file_widget = gtk_check_button_new_with_label ("C");
g_signal_connect (G_OBJECT (c_file_widget), "clicked",
            G_CALLBACK(c_file_widget_clicked), this);

gtk_grid_attach (GTK_GRID(grid), c_file_widget, 0, 5, 20, 5); // L, T, W, H


                   
c_c_type_checkbox = gtk_check_button_new_with_label("/**/");
g_signal_connect (G_OBJECT (c_c_type_checkbox), "clicked",
            G_CALLBACK(c_c_type_checkbox_clicked), this);


gtk_grid_attach (GTK_GRID(grid), c_c_type_checkbox, 20, 5, 11, 5); // L, T, W, H



if (rh->flags.c_file)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (c_file_widget), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(c_c_type_checkbox), TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (c_c_type_checkbox), TRUE);
  }
else
  {
  gtk_widget_set_sensitive(GTK_WIDGET(c_c_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (c_c_type_checkbox), FALSE);
	}


// ------  CPP horizontally ----------------------------------------------------
cpp_file_widget = gtk_check_button_new_with_label("CPP");
g_signal_connect (G_OBJECT (cpp_file_widget), "clicked",
            G_CALLBACK(cpp_file_widget_clicked), this);

gtk_grid_attach (GTK_GRID(grid), cpp_file_widget, 0, 10, 20, 5); // L, T, W, H

  
cpp_c_type_checkbox = gtk_check_button_new_with_label("/**/");
g_signal_connect (G_OBJECT (cpp_c_type_checkbox), "clicked",
            G_CALLBACK(cpp_type_checkbox_clicked), this);


gtk_grid_attach (GTK_GRID(grid), cpp_c_type_checkbox, 20, 10, 11, 5); // L, T, W, H


  
cpp_cpp_type_checkbox = gtk_check_button_new_with_label("//**"); 
g_signal_connect (G_OBJECT (cpp_cpp_type_checkbox), "clicked",
            G_CALLBACK(cpp_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), cpp_cpp_type_checkbox, 31, 10, 11, 5); // L, T, W, H

 

cpp2_cpp_type_checkbox = gtk_check_button_new_with_label("////"); 
g_signal_connect (G_OBJECT (cpp2_cpp_type_checkbox), "clicked",
            G_CALLBACK(cpp_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), cpp2_cpp_type_checkbox, 42, 10, 11, 5); // L, T, W, H



if (rh->flags.cpp_file)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_file_widget), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(cpp_cpp_type_checkbox), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(cpp2_cpp_type_checkbox), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(cpp_c_type_checkbox), TRUE);      
  if (rh->flags.cpp_t)
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_c_type_checkbox), FALSE);			
    gtk_toggle_button_set_active	(GTK_TOGGLE_BUTTON (cpp2_cpp_type_checkbox), FALSE);		
    gtk_toggle_button_set_active	(GTK_TOGGLE_BUTTON (cpp_cpp_type_checkbox), TRUE);		
    }
  else if (rh->flags.cpp2_t)
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_c_type_checkbox), FALSE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp2_cpp_type_checkbox), TRUE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_cpp_type_checkbox), FALSE);
		}
  else
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_c_type_checkbox), TRUE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_cpp_type_checkbox), FALSE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp2_cpp_type_checkbox), FALSE);
		}
  }
else
  {
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_file_widget), FALSE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_c_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cpp_cpp_type_checkbox), FALSE);
			
  gtk_widget_set_sensitive(GTK_WIDGET(cpp_c_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(cpp_cpp_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(cpp2_cpp_type_checkbox), FALSE);
  }



// ------  ADA horizontally ----------------------------------------------------

ada_file_widget = gtk_check_button_new_with_label ("ADA");
g_signal_connect (G_OBJECT (ada_file_widget), "clicked",
            G_CALLBACK(ada_file_widget_clicked), this);
if (rh->flags.ada_file)
	{
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ada_file_widget), TRUE);
	}
	

gtk_grid_attach (GTK_GRID(grid), ada_file_widget, 0, 15, 20, 5); // L, T, W, H


ada_type_checkbox = gtk_check_button_new_with_label("--");
g_signal_connect (G_OBJECT (ada_type_checkbox), "clicked",
            G_CALLBACK(ada_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), ada_type_checkbox, 20, 15, 11, 5); // L, T, W, H


if (rh->flags.ada_file)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ada_file_widget), TRUE);	
  gtk_widget_set_sensitive(GTK_WIDGET(ada_type_checkbox), TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ada_type_checkbox), TRUE);
	}
else
  {
  gtk_widget_set_sensitive(GTK_WIDGET(ada_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ada_type_checkbox), FALSE);
	}

 
// ------  PHP horizontally ----------------------------------------------------  
php_file_widget = gtk_check_button_new_with_label ("PHP");
g_signal_connect (G_OBJECT (php_file_widget), "clicked",
            G_CALLBACK(php_file_widget_clicked), this);

gtk_grid_attach (GTK_GRID(grid), php_file_widget, 0, 20, 20, 5); // L, T, W, H


php_c_type_checkbox = gtk_check_button_new_with_label("/**/");
g_signal_connect (G_OBJECT (php_c_type_checkbox), "clicked",
            G_CALLBACK(php_type_checkbox_clicked), this);
 
gtk_grid_attach (GTK_GRID(grid), php_c_type_checkbox, 20, 20, 11, 5); // L, T, W, H

 
php_cpp_type_checkbox = gtk_check_button_new_with_label("//**"); 
g_signal_connect (G_OBJECT (php_cpp_type_checkbox), "clicked",
            G_CALLBACK(php_type_checkbox_clicked), this);
 
gtk_grid_attach (GTK_GRID(grid), php_cpp_type_checkbox, 31, 20, 11, 5); // L, T, W, H


                                                        
php_cpp2_type_checkbox = gtk_check_button_new_with_label("////"); 
g_signal_connect (G_OBJECT (php_cpp2_type_checkbox), "clicked",
            G_CALLBACK(php_type_checkbox_clicked), this);
 
gtk_grid_attach (GTK_GRID(grid), php_cpp2_type_checkbox, 42, 20, 11, 5); // L, T, W, H


                                                        

php_hash_type_checkbox = gtk_check_button_new_with_label("##"); 
g_signal_connect (G_OBJECT (php_hash_type_checkbox), "clicked",
            G_CALLBACK(php_type_checkbox_clicked), this);
 
gtk_grid_attach (GTK_GRID(grid), php_hash_type_checkbox, 53, 20, 11, 5); // L, T, W, H



if (rh->flags.php_file)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_file_widget), TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_c_type_checkbox), TRUE);
			
  gtk_widget_set_sensitive(GTK_WIDGET(php_cpp_type_checkbox), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(php_cpp2_type_checkbox), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(php_hash_type_checkbox), TRUE);
  if (rh->flags.php_cpp_t)
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp2_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_c_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_hash_type_checkbox), FALSE);
		}
  if (rh->flags.php_cpp2_t)
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp2_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_c_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_hash_type_checkbox), FALSE);
		}
  else if (rh->flags.php_h_t)
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_hash_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_c_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp2_type_checkbox), FALSE);
		}
  else // DEFAULT
    {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_c_type_checkbox), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp2_type_checkbox), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_hash_type_checkbox), FALSE);
		}
  }
else
  {
  gtk_widget_set_sensitive(GTK_WIDGET(php_c_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(php_cpp_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(php_cpp2_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(php_hash_type_checkbox), FALSE);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_c_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_cpp2_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_hash_type_checkbox), FALSE);
	}

// ------ JAVA horizontally ----------------------------------------------------
java_file_widget = gtk_check_button_new_with_label ("JAVA");
g_signal_connect (G_OBJECT (java_file_widget), "clicked",
            G_CALLBACK(java_file_widget_clicked), this);
 
if (rh->flags.java_file)
	{
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_file_widget), TRUE);
	}

gtk_grid_attach (GTK_GRID(grid), java_file_widget, 0, 25, 20, 5); // L, T, W, H



                   
java_c_type_checkbox = gtk_check_button_new_with_label("/**/");
g_signal_connect (G_OBJECT (java_c_type_checkbox), "clicked",
            G_CALLBACK(java_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), java_c_type_checkbox, 20, 25, 11, 5); // L, T, W, H

  
java_cpp_type_checkbox = gtk_check_button_new_with_label("//**"); 
g_signal_connect (G_OBJECT (java_cpp_type_checkbox), "clicked",
            G_CALLBACK(java_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), java_cpp_type_checkbox, 31, 25, 11, 5); // L, T, W, H



java_cpp2_type_checkbox = gtk_check_button_new_with_label("////"); 
g_signal_connect (G_OBJECT (java_cpp2_type_checkbox), "clicked",
            G_CALLBACK(java_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), java_cpp2_type_checkbox, 42, 25, 11, 5); // L, T, W, H


if (rh->flags.java_file)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_file_widget), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(java_c_type_checkbox), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(java_cpp_type_checkbox), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(java_cpp2_type_checkbox), TRUE);

  if (rh->flags.java_cpp_t)
		{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_cpp_type_checkbox), TRUE);
		}
  else if (rh->flags.java_cpp2_t)
		{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_cpp2_type_checkbox), TRUE);
		}
  else
		{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_c_type_checkbox), TRUE);
		}
  }
else
  {
  gtk_widget_set_sensitive(GTK_WIDGET(java_c_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(java_cpp_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(java_cpp2_type_checkbox), FALSE);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_c_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_cpp_type_checkbox), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_cpp2_type_checkbox), FALSE);
	}


// ------ PERL horizontally ----------------------------------------------------
perl_file_widget = gtk_check_button_new_with_label ("PERL");
g_signal_connect (G_OBJECT (perl_file_widget), "clicked",
            G_CALLBACK(perl_file_widget_clicked), this);

gtk_grid_attach (GTK_GRID(grid), perl_file_widget, 0, 30, 20, 5); // L, T, W, H



perl_hash_type_checkbox = gtk_check_button_new_with_label("##"); 
g_signal_connect (G_OBJECT (perl_hash_type_checkbox), "clicked",
           G_CALLBACK(perl_hash_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), perl_hash_type_checkbox, 20, 30, 11, 5); // L, T, W, H



if (rh->flags.perl_file)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (perl_file_widget), TRUE);
	}
else
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (perl_file_widget), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (perl_hash_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(perl_hash_type_checkbox), FALSE);
  }

// ------ FORTRAN horizontally ------------------------------------------------- 
fortran_file_widget = gtk_check_button_new_with_label ("FORTRAN 90/95");
g_signal_connect (G_OBJECT (fortran_file_widget), "clicked",
            G_CALLBACK(fortran_file_widget_clicked), this);

gtk_grid_attach (GTK_GRID(grid), fortran_file_widget, 0, 35, 20, 5); // L, T, W, H



fortran_type_checkbox = gtk_check_button_new_with_label("!"); 
g_signal_connect (G_OBJECT (fortran_type_checkbox), "clicked",
           G_CALLBACK(fortran_type_checkbox_clicked), this);

gtk_grid_attach (GTK_GRID(grid), fortran_type_checkbox, 20, 35, 11, 5); // L, T, W, H


if (rh->flags.fortran_file)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (fortran_file_widget), TRUE);
  }
else
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (fortran_file_widget), FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (fortran_type_checkbox), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(fortran_type_checkbox), FALSE);
  }


//==========================================================================

box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
gtk_box_set_homogeneous (GTK_BOX(box), FALSE);

gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 0);


frame = gtk_frame_new("RH.log");                   
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 0);


grid = gtk_grid_new ();
gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
                   
gtk_container_add(GTK_CONTAINER(frame), grid);  
gtk_container_set_border_width(GTK_CONTAINER(grid), 10);


gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
       "Log File\nChoose which type of log file, RH.log, is required (or none)");
                            
	
wid = gtk_label_new("NONE");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);    
                   
gtk_grid_attach (GTK_GRID(grid), wid, 6, 2, 14, 5);
                   
wid = gtk_label_new("OVERWRITE");   
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);    
                   
gtk_grid_attach (GTK_GRID(grid), wid, 20, 2, 24, 5);  // L, T, W, H
                   


wid = gtk_label_new("APPEND");   
gtk_misc_set_alignment (GTK_MISC (wid), .5, .5);    
                   
gtk_grid_attach (GTK_GRID(grid), wid, 44, 2, 20, 5);  // L, T, W, H
                   


none_log_checkbox =  gtk_check_button_new();
g_signal_connect (G_OBJECT (none_log_checkbox), "clicked",
            G_CALLBACK(log_checkbox_clicked), this);
                   
gtk_grid_attach (GTK_GRID(grid), none_log_checkbox, 8, 8, 12, 5);  // L, T, W, H
                   


overwrite_log_checkbox =  gtk_check_button_new(); 
g_signal_connect (G_OBJECT (overwrite_log_checkbox), "clicked",
            G_CALLBACK(log_checkbox_clicked), this);
                   
gtk_grid_attach (GTK_GRID(grid), overwrite_log_checkbox, 31, 8, 12, 5);  // L, T, W, H
                   

append_log_checkbox =  gtk_check_button_new();
g_signal_connect (G_OBJECT (append_log_checkbox), "clicked",
            G_CALLBACK(log_checkbox_clicked), this);
                   
gtk_grid_attach (GTK_GRID(grid), append_log_checkbox, 52, 8, 12, 5);  // L, T, W, H
                   
  
if (rh->flags.append_log)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(append_log_checkbox), TRUE);
	}
else if (rh->flags.want_log)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(overwrite_log_checkbox), TRUE);
	}
else
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(none_log_checkbox), TRUE);
	}

log2cerr_checkbox = gtk_check_button_new_with_label("Log also to cerr");
if (rh->flags.log_to_cerr)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(log2cerr_checkbox), TRUE);
	}
                   
gtk_grid_attach (GTK_GRID(grid), log2cerr_checkbox, 22, 14, 42, 5);  // L, T, W, H
                   

}

void Environment::callback_version_clicked(GtkWidget *widget, Environment *ev)
{
GtkWidget *clear_widget, *browse_widget, *entry_widget;

if (ev->c_path_entry && ev->c_clear_button && ev->c_browse_button &&
    ev->ada_path_entry && ev->ada_clear_button && ev->ada_browse_button &&
    ev->php_path_entry && ev->php_clear_button && ev->php_browse_button &&
    ev->java_path_entry && ev->java_clear_button && ev->java_browse_button &&
    ev->perl_path_entry && ev->perl_clear_button && ev->perl_browse_button &&
    ev->fortran_path_entry && ev->fortran_clear_button && ev->fortran_browse_button)    
  {
  if (widget == ev->c_version_checkbox)
    {
    entry_widget  = ev->c_path_entry;
    clear_widget  = ev->c_clear_button;
    browse_widget = ev->c_browse_button;
    }
  else if (widget == ev->ada_version_checkbox)
    {
    entry_widget  = ev->ada_path_entry;
    clear_widget  = ev->ada_clear_button;
    browse_widget = ev->ada_browse_button;
    }
  else if (widget == ev->php_version_checkbox)
    {
    entry_widget  = ev->php_path_entry;
    clear_widget  = ev->php_clear_button;
    browse_widget = ev->php_browse_button;
    }
  else if (widget == ev->java_version_checkbox)
    {
    entry_widget  = ev->java_path_entry;
    clear_widget  = ev->java_clear_button;
    browse_widget = ev->java_browse_button;
    }
  else if (widget == ev->perl_version_checkbox)
    {
    entry_widget  = ev->perl_path_entry;
    clear_widget  = ev->perl_clear_button;
    browse_widget = ev->perl_browse_button;
    }
  else if (widget == ev->fortran_version_checkbox)
    {
    entry_widget  = ev->fortran_path_entry;
    clear_widget  = ev->fortran_clear_button;
    browse_widget = ev->fortran_browse_button;
    }
  else return;
  }
else return;

if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
  {
  gtk_widget_set_sensitive(GTK_WIDGET(entry_widget), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(clear_widget), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(browse_widget), TRUE);
  }
else
  {
  gtk_widget_set_sensitive(GTK_WIDGET(entry_widget), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(clear_widget), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(browse_widget), FALSE);
  }
}


// ==============
// VERSION PAGE
// ==============


void Environment::callback_want_micro_clicked(GtkWidget *widget,  Environment *ev)
{
//cout << "Environment::callback_want_micro_clicked" << endl;

if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ev->p_want_micro_checkbutton)))
  {
  gtk_button_set_label(GTK_BUTTON(ev->p_auto_inc_checkbutton), "Auto-increment PROJECT micro version");
  }
else
  {
  gtk_button_set_label(GTK_BUTTON(ev->p_auto_inc_checkbutton), "Auto-increment PROJECT minor version");
  }
}


void Environment::fill_version_page(GtkWidget *vbox)
{
GtkWidget *wid, *grid, *frame, *hbox, *box;
	
frame = gtk_frame_new("Version Preferences");
gtk_container_set_border_width(GTK_CONTAINER(frame), 10);


gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);


grid = gtk_grid_new();
gtk_container_set_border_width(GTK_CONTAINER(grid), 20);


gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
gtk_grid_set_row_spacing (GTK_GRID(grid), 5);
gtk_container_add(GTK_CONTAINER(frame), grid);  


  

// VERSION INFO
wid = gtk_label_new("Language");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);
	

gtk_grid_attach (GTK_GRID(grid), wid, 0, 0, 8, 1);
	
wid = gtk_label_new("Version File");  
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 8, 0, 7, 1);

wid = gtk_label_new("Directory");  
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);

gtk_grid_attach (GTK_GRID(grid), wid, 22, 0, 18, 1);


// ------ C / C++ horizontally -------------------------------------------------

wid = gtk_label_new("C / C++");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);
gtk_grid_attach (GTK_GRID(grid), wid, 0, 1, 9, 1);


	
c_version_checkbox = gtk_check_button_new();
g_signal_connect (G_OBJECT (c_version_checkbox), "clicked",
            G_CALLBACK(callback_version_clicked), this);

gtk_widget_set_tooltip_text (GTK_WIDGET(c_version_checkbox),
    "C / C++ version file will be written if selected");


gtk_grid_attach (GTK_GRID(grid), c_version_checkbox, 9, 1, 3, 1); // L, T, W, H


c_path_entry = gtk_entry_new();
gtk_editable_set_editable(GTK_EDITABLE(c_path_entry), FALSE);
	
gtk_entry_set_text(GTK_ENTRY(c_path_entry), rh->c_version_path.c_str());

gtk_widget_set_tooltip_text (GTK_WIDGET(c_path_entry),
    "Path for C / C++ version file. If empty,the version file\n"
    "  appears in the first scanned directory of this language type.\n"
    "  It defaults to the project directory if none found");

gtk_grid_attach (GTK_GRID(grid), c_path_entry, 12, 1, 20, 1); // L, T, W, H


c_clear_button = gtk_button_new();
decorate_widget(c_clear_button, &black, NULL, NULL, NULL, GTK_STOCK_CLEAR);

gtk_widget_set_tooltip_text (GTK_WIDGET(c_clear_button),
    "Clear this entry");
g_signal_connect (G_OBJECT (c_clear_button), "clicked",
            G_CALLBACK(callback_delete_c), this);

gtk_grid_attach (GTK_GRID(grid), c_clear_button, 32, 1, 3, 1); // L, T, W, H



c_browse_button = gtk_button_new_with_label("Browse");

decorate_widget(c_browse_button, &black, NULL, NULL, NULL, NULL);

gtk_widget_set_tooltip_text (GTK_WIDGET(c_browse_button),
    "Browse to select path for version file");
g_signal_connect (G_OBJECT (c_browse_button), "clicked",
            G_CALLBACK(callback_browse_c), this);

gtk_grid_attach (GTK_GRID(grid), c_browse_button, 35, 1, 5, 1); // L, T, W, H


if (rh->flags.c_v)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (c_version_checkbox), TRUE);
  }
else
  {
  gtk_widget_set_sensitive(c_path_entry, FALSE);
  gtk_widget_set_sensitive(c_clear_button, FALSE);
  gtk_widget_set_sensitive(c_browse_button, FALSE);
  }



// ------ ADA horizontally -----------------------------------------------------
 
wid = gtk_label_new("ADA");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);  
gtk_grid_attach (GTK_GRID(grid), wid, 0, 2, 9, 1); // L, T, W, H

ada_version_checkbox = gtk_check_button_new(); 
g_signal_connect (G_OBJECT (ada_version_checkbox), "clicked",
            G_CALLBACK(callback_version_clicked), this);

gtk_widget_set_tooltip_text (GTK_WIDGET(ada_version_checkbox),
     "ADA version file will be written if selected");

gtk_grid_attach (GTK_GRID(grid), ada_version_checkbox, 9, 2, 3, 1); // L, T, W, H


ada_path_entry = gtk_entry_new();
gtk_editable_set_editable(GTK_EDITABLE(ada_path_entry), FALSE);
	
gtk_entry_set_text(GTK_ENTRY(ada_path_entry), rh->ada_version_path.c_str());

gtk_widget_set_tooltip_text (GTK_WIDGET(ada_path_entry),
    "Path for ADA version file. If empty,the version file\n"
    "  appears in the first scanned directory of this language type.\n"
    "  It defaults to the project directory if none found");

gtk_grid_attach (GTK_GRID(grid), ada_path_entry, 12, 2, 20, 1); // L, T, W, H


ada_clear_button = gtk_button_new();
decorate_widget(ada_clear_button, &black, NULL, NULL, NULL, GTK_STOCK_CLEAR);

gtk_widget_set_tooltip_text (GTK_WIDGET(ada_clear_button),
    "Clear this entry");
g_signal_connect (G_OBJECT (ada_clear_button), "clicked",
            G_CALLBACK (callback_delete_ada), this);

gtk_grid_attach (GTK_GRID(grid), ada_clear_button, 32, 2, 3, 1); // L, T, W, H


ada_browse_button = gtk_button_new_with_label("Browse");
decorate_widget(ada_browse_button, &black, NULL, NULL, NULL, NULL);


gtk_widget_set_tooltip_text (GTK_WIDGET(ada_browse_button),
    "Browse to select path for version file");
g_signal_connect (G_OBJECT (ada_browse_button), "clicked",
            G_CALLBACK (callback_browse_ada), this);

gtk_grid_attach (GTK_GRID(grid), ada_browse_button, 35, 2, 5, 1); // L, T, W, H


if (rh->flags.ada_v)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ada_version_checkbox), TRUE);
  }
else
  {
  gtk_widget_set_sensitive(ada_path_entry, FALSE);
  gtk_widget_set_sensitive(ada_clear_button, FALSE);
  gtk_widget_set_sensitive(ada_browse_button, FALSE);
  }
  
// ------ PHP horizontally -----------------------------------------------------

wid = gtk_label_new("PHP");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);  

gtk_grid_attach (GTK_GRID(grid), wid, 0, 3, 9, 1); // L, T, W, H


php_version_checkbox = gtk_check_button_new(); 
g_signal_connect (G_OBJECT (php_version_checkbox), "clicked",
            G_CALLBACK(callback_version_clicked), this);

gtk_widget_set_tooltip_text (GTK_WIDGET(php_version_checkbox),
    "PHP version file will be written if selected");

gtk_grid_attach (GTK_GRID(grid), php_version_checkbox, 9, 3, 3, 1); // L, T, W, H


php_path_entry = gtk_entry_new();
gtk_editable_set_editable(GTK_EDITABLE(php_path_entry), FALSE);
	
gtk_entry_set_text(GTK_ENTRY(php_path_entry), rh->php_version_path.c_str());

gtk_widget_set_tooltip_text (GTK_WIDGET(php_path_entry),
    "Path for PHP version file. If empty,the version file\n"
    "  appears in the first scanned directory of this language type.\n"
    "  It defaults to the project directory if none found");

gtk_grid_attach (GTK_GRID(grid), php_path_entry, 12, 3, 20, 1); // L, T, W, H


php_clear_button = gtk_button_new();
decorate_widget(php_clear_button, &black, NULL, NULL, NULL, GTK_STOCK_CLEAR);


gtk_widget_set_tooltip_text (GTK_WIDGET(php_clear_button),
    "Clear this entry");
g_signal_connect (G_OBJECT (php_clear_button), "clicked",
            G_CALLBACK(callback_delete_php), this);

gtk_grid_attach (GTK_GRID(grid), php_clear_button, 32, 3, 3, 1); // L, T, W, H



php_browse_button = gtk_button_new_with_label("Browse");
decorate_widget(php_browse_button, &black, NULL, NULL, NULL, NULL);


gtk_widget_set_tooltip_text (GTK_WIDGET(php_browse_button),
    "Browse to select path for version file");
g_signal_connect (G_OBJECT (php_browse_button), "clicked",
            G_CALLBACK(callback_browse_php), this);

gtk_grid_attach (GTK_GRID(grid), php_browse_button, 35, 3, 5, 1); // L, T, W, H



if (rh->flags.php_v)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (php_version_checkbox), TRUE);
  }
else
  {
  gtk_widget_set_sensitive(php_path_entry, FALSE);
  gtk_widget_set_sensitive(php_clear_button, FALSE);
  gtk_widget_set_sensitive(php_browse_button, FALSE);
  }


// ------ JAVA horizontally ---------------------------------------------------- 

wid = gtk_label_new("JAVA");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);  

gtk_grid_attach (GTK_GRID(grid), wid, 0, 4, 9, 1); // L, T, W, H

  
java_version_checkbox = gtk_check_button_new(); 
g_signal_connect (G_OBJECT (java_version_checkbox), "clicked",
            G_CALLBACK(callback_version_clicked), this);
gtk_widget_set_tooltip_text (GTK_WIDGET(java_version_checkbox),
    "JAVA version file will be written if selected");

gtk_grid_attach (GTK_GRID(grid), java_version_checkbox, 9, 4, 3, 1); // L, T, W, H


java_path_entry = gtk_entry_new();
gtk_editable_set_editable(GTK_EDITABLE(java_path_entry), FALSE);
gtk_entry_set_text(GTK_ENTRY(java_path_entry), rh->java_version_path.c_str());
gtk_widget_set_tooltip_text (GTK_WIDGET(java_path_entry),
    "Path for JAVA version file. If empty,the version file\n"
    "  appears in the first scanned directory of this language type.\n"
    "  It defaults to the project directory if none found");

gtk_grid_attach (GTK_GRID(grid), java_path_entry, 12, 4, 20, 1); // L, T, W, H



java_clear_button = gtk_button_new();
decorate_widget(java_clear_button, &black, NULL, NULL, NULL, GTK_STOCK_CLEAR);

gtk_widget_set_tooltip_text (GTK_WIDGET(java_clear_button),
    "Clear this entry");
g_signal_connect (G_OBJECT (java_clear_button), "clicked",
            G_CALLBACK(callback_delete_java), this);

gtk_grid_attach (GTK_GRID(grid), java_clear_button, 32, 4, 3, 1); // L, T, W, H



java_browse_button = gtk_button_new_with_label("Browse");
decorate_widget(java_browse_button, &black, NULL, NULL, NULL, NULL);

gtk_widget_set_tooltip_text (GTK_WIDGET(java_browse_button),
    "Browse to select path for version file");
g_signal_connect (G_OBJECT (java_browse_button), "clicked",
            G_CALLBACK(callback_browse_java), this);

gtk_grid_attach (GTK_GRID(grid), java_browse_button, 35, 4, 5, 1); // L, T, W, H



if (rh->flags.java_v)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (java_version_checkbox), TRUE);
	}
else
  {
  gtk_widget_set_sensitive(java_path_entry, FALSE);
  gtk_widget_set_sensitive(java_clear_button, FALSE);
  gtk_widget_set_sensitive(java_browse_button, FALSE);
  }

// ------ PERL horizontally ----------------------------------------------------

wid = gtk_label_new("PERL");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);  

gtk_grid_attach (GTK_GRID(grid), wid, 0, 5, 9, 1); // L, T, W, H


perl_version_checkbox = gtk_check_button_new(); 
g_signal_connect (G_OBJECT (perl_version_checkbox), "clicked",
            G_CALLBACK(callback_version_clicked), this);
gtk_widget_set_tooltip_text (GTK_WIDGET(perl_version_checkbox),
    "PERL version file will be written if selected");

gtk_grid_attach (GTK_GRID(grid), perl_version_checkbox, 9, 5, 3, 1); // L, T, W, H


perl_path_entry = gtk_entry_new();
gtk_entry_set_text(GTK_ENTRY(perl_path_entry), rh->perl_version_path.c_str());
gtk_editable_set_editable(GTK_EDITABLE(perl_path_entry), FALSE);
gtk_widget_set_tooltip_text (GTK_WIDGET(perl_path_entry),
    "Path for PERL version file. If empty,the version file\n"
    "  appears in the first scanned directory of this language type.\n"
    "  It defaults to the project directory if none found");

gtk_grid_attach (GTK_GRID(grid), perl_path_entry, 12, 5, 20, 1); // L, T, W, H



perl_clear_button = gtk_button_new();
decorate_widget(perl_clear_button, &black, NULL, NULL, NULL, GTK_STOCK_CLEAR);


gtk_widget_set_tooltip_text (GTK_WIDGET(perl_clear_button),
   "Clear this entry");
g_signal_connect (G_OBJECT (perl_clear_button), "clicked",
            G_CALLBACK(callback_delete_perl), this);

gtk_grid_attach (GTK_GRID(grid), perl_clear_button, 32, 5, 3, 1); // L, T, W, H




perl_browse_button = gtk_button_new_with_label("Browse");
decorate_widget(perl_browse_button, &black, NULL, NULL, NULL, NULL);


gtk_widget_set_tooltip_text (GTK_WIDGET(perl_browse_button),
    "Browse to select path for version file");

g_signal_connect (G_OBJECT (perl_browse_button), "clicked",
            G_CALLBACK(callback_browse_perl), this);

gtk_grid_attach (GTK_GRID(grid), perl_browse_button, 35, 5, 5, 1); // L, T, W, H



if (rh->flags.perl_v)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (perl_version_checkbox), TRUE);
	}
else
  {
  gtk_widget_set_sensitive(perl_path_entry, FALSE);
  gtk_widget_set_sensitive(perl_clear_button, FALSE);
  gtk_widget_set_sensitive(perl_browse_button, FALSE);
  }



// ------ FORTRAN horizontally -------------------------------------------------

wid = gtk_label_new("FORTRAN 90/95");
gtk_misc_set_alignment (GTK_MISC (wid), 0, .5);  

gtk_grid_attach (GTK_GRID(grid), wid, 0, 6, 9, 1); // L, T, W, H



fortran_version_checkbox = gtk_check_button_new(); 
g_signal_connect (G_OBJECT (fortran_version_checkbox), "clicked",
            G_CALLBACK(callback_version_clicked), this);
gtk_widget_set_tooltip_text (GTK_WIDGET(fortran_version_checkbox),
   "FORTRAN version file will be written if selected");

gtk_grid_attach (GTK_GRID(grid), fortran_version_checkbox, 9, 6, 3, 1); // L, T, W, H



fortran_path_entry = gtk_entry_new();
gtk_entry_set_text(GTK_ENTRY(fortran_path_entry), rh->fortran_version_path.c_str());
gtk_editable_set_editable(GTK_EDITABLE(fortran_path_entry), FALSE);
gtk_widget_set_tooltip_text (GTK_WIDGET(fortran_path_entry),
    "Path for FORTRAN version file. If empty,the version file\n"
    "  appears in the first scanned directory of this language type.\n"
    "  It defaults to the project directory if none found");

gtk_grid_attach (GTK_GRID(grid), fortran_path_entry, 12, 6, 20, 1); // L, T, W, H



fortran_clear_button = gtk_button_new();
decorate_widget(fortran_clear_button, &black, NULL, NULL, NULL, GTK_STOCK_CLEAR);


gtk_widget_set_tooltip_text (GTK_WIDGET(fortran_clear_button),
    "Clear this entry");
g_signal_connect (G_OBJECT (fortran_clear_button), "clicked",
            G_CALLBACK(callback_delete_fortran), this);

gtk_grid_attach (GTK_GRID(grid), fortran_clear_button, 32, 6, 3, 1); // L, T, W, H



fortran_browse_button = gtk_button_new_with_label("Browse");
decorate_widget(fortran_browse_button, &black, NULL, NULL, NULL, NULL);

gtk_widget_set_tooltip_text (GTK_WIDGET(fortran_browse_button),
    "Browse to select path for version file");
g_signal_connect (G_OBJECT (fortran_browse_button), "clicked",
            G_CALLBACK(callback_browse_fortran), this);

gtk_grid_attach (GTK_GRID(grid), fortran_browse_button, 35, 6, 5, 1); // L, T, W, H



if (rh->flags.fortran_v)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (fortran_version_checkbox), TRUE);
	}
else
  {
  gtk_widget_set_sensitive(fortran_path_entry, FALSE);
  gtk_widget_set_sensitive(fortran_clear_button, FALSE);
  gtk_widget_set_sensitive(fortran_browse_button, FALSE);
  }
	
hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);


	
// PROJECT VERSION OPTIONS

frame = gtk_frame_new("PROJECT Version Options");
  
gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "PROJECT Version Options\n"
      "Auto-increment PROJECT - select this if project versions should auto-increment.\n"
      "    The version can be altered manually in any case.");                     

gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 5);
	
box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
gtk_container_add (GTK_CONTAINER(frame), box);
                             
p_want_micro_checkbutton = gtk_check_button_new_with_label("Want Project MICRO version");
if (existing_project_flag)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (p_want_micro_checkbutton), rh->flags.p_want_micro);
	}
else
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (p_want_micro_checkbutton), rh->flags.np_want_micro);
	}

g_signal_connect (G_OBJECT (p_want_micro_checkbutton), "clicked",
            G_CALLBACK(callback_want_micro_clicked), this);

gtk_box_pack_start(GTK_BOX(box), p_want_micro_checkbutton, TRUE, TRUE, 5);


    // AUTO-INCREMENT PROJECT VERSION
p_auto_inc_checkbutton = gtk_check_button_new_with_label(
      (rh->flags.p_want_micro) ? 
      "Auto-increment PROJECT micro version" : 
      "Auto-increment PROJECT minor version" );

if (existing_project_flag)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (p_auto_inc_checkbutton), rh->flags.p_auto_inc);
	}
else
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (p_auto_inc_checkbutton), rh->flags.np_auto_inc);
	}

gtk_box_pack_start(GTK_BOX(box), p_auto_inc_checkbutton, TRUE, TRUE, 5);







    
// =====================================================
// This should be part of the individual file processing
// =====================================================
frame = gtk_frame_new("FILE Version Options");

gtk_widget_set_tooltip_text (GTK_WIDGET(frame),
      "FILE Version Options\n"
      "Auto-increment FILE - if file versions are required, select this for\n"
      "   auto-incrementing file versions when a file has changed.\n"
      "   As with the project version, this can also be altered manually.\n"
      "NOTE:\n"
      "   File versions are selected for use on an individual file basis.\n"
      "   During file processing the user can opt for file versions.");

gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 5);
	
box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
gtk_container_add (GTK_CONTAINER(frame), box);
                             

// AUTO-INCREMENT FILE VERSIONS
f_auto_inc_checkbutton = gtk_check_button_new_with_label("Auto-increment FILE micro versions");

gtk_box_pack_start(GTK_BOX(box), f_auto_inc_checkbutton, TRUE, TRUE, 5);


                                                          
if (rh->flags.f_auto_inc)
  {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (f_auto_inc_checkbutton), TRUE);
  }

}



/****************************************
 * void GetProjectEnv()                 *
 * Get the project environment          *
 ****************************************/

void Environment::get_env(gboolean existing_flag)
{
GtkWidget *widget;
GtkWidget *hbox;
GtkWidget *frame;
string buffer;
Date dt(rh);
	
buffer = "GET PROJECT ENVIRONMENT - ";
buffer += (existing_flag)  ? "EXISTING" : "NEW";
buffer +=  " PROJECT - " + ( rh->get_project_name_and_version() );
dt.date_log_operation(&buffer);

	
// If this is a new project, various rh->flags need to be cleared
existing_project_flag = existing_flag;

rh->DestroyWidgets();

gtk_label_set_text (GTK_LABEL (rh->title), buffer.c_str() );

buffer = "Project Path: \"" + rh->project_path +  "\"";
gtk_label_set_text (GTK_LABEL (rh->status), buffer.c_str() );




main_grid = gtk_grid_new ();
gtk_container_add (GTK_CONTAINER (rh->vbox), main_grid);
gtk_grid_set_column_homogeneous (GTK_GRID(main_grid), TRUE);
	
notebook = gtk_notebook_new();  
gtk_notebook_set_tab_pos (GTK_NOTEBOOK(notebook), GTK_POS_TOP);

gtk_grid_attach (GTK_GRID(main_grid), notebook, 0, 0, 64, 72); // L, T, W, H

	
widget = add_page(notebook, (gchar *)"Project");
fill_project_page(widget);

widget = add_page(notebook, (gchar *)"Copyright");
fill_copyright_page(widget);

widget = add_page(notebook, (gchar *)"Languages & Log");
fill_language_page(widget);

widget = add_page(notebook, (gchar *)"Version Files");
fill_version_page(widget);

cout << "Environment::get_env - 1" << endl;

widget = add_page(notebook, (gchar *)"Directories and ChangLog");
cout << "Environment::get_env - 2" << endl;	
fill_directories_page(widget);


cout << "Environment::get_env - 3" << endl;

// ------  Add OK & CANCEL & QuitAll horizontally
hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
gtk_box_set_homogeneous (GTK_BOX(hbox), FALSE);
gtk_grid_attach (GTK_GRID(main_grid), hbox, 0, 72, 64, 8); // L, T, W, H


widget = CreateButton(hbox, (char *)RevHead::QuitAllTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_QUIT);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(abort_func), this);

widget = CreateButton(hbox, (char *)RevHead::CancelTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_CANCEL);
	
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(cancel_func), this);

widget = CreateButton(hbox, (char *)"SAVE" );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_SAVE);
g_signal_connect (G_OBJECT (widget), "clicked",
        G_CALLBACK(env_OK_func), this);

gtk_widget_show_all(rh->vbox);
}


