/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       fixfiles.cpp                                              *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       15-Apr-2013 16.44.30 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Functions concerned with TOUCH and FIXDATE                                *
* 20-Aug-2004 12.10.29 -36000 0.4     David M. Balean                       *
* Very similar to C version                                                 *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/
#include <string.h>

#include "fixfiles.h"

#include "logfile.h"
#include "complain.h"
#include "date.h"
#include "misc.h"
#include "quit.h"

using namespace std;
using namespace misc;

string Fixfiles::FixDateTXT  = "FIXDATE FILE(S)";
string Fixfiles::TouchTXT    = "TOUCH FILE(S)";
gboolean Fixfiles::adding_list = FALSE;
gint Fixfiles::add_timer = 0;
gint Fixfiles::rem_timer = 0;

list<string>  Fixfiles::pathname_list;


Fixfiles::Fixfiles(RevHead *revhead)
{
cout << "Fixfiles:" << endl;

rh = revhead;
fixdate = adding_list = FALSE;
rem_timer = add_timer = 0;
}

Fixfiles::~Fixfiles(void)
{
cout << "~Fixfiles:" << endl;

if (rem_timer) g_source_remove(rem_timer);  //gtk_timeout_remove(rem_timer);
rem_timer = 0;

if (add_timer) g_source_remove(add_timer);  //gtk_timeout_remove(add_timer);
add_timer = 0;
}


void Fixfiles::QuitFixfiles(GtkWidget *widget, Fixfiles *ff)
{
ff->rh->FrontPanel();
}

void Fixfiles::callback_continue(GtkWidget *widget, class Complain *cn)
{
RevHead *rh = (RevHead *)(cn->data);
cn->end_complain(widget, cn);
rh->FrontPanel();
}

void Fixfiles::do_fixfiles(GtkWidget *widget, Fixfiles *ff)
{
int type;
time_t ds;
time_t tz;
int daylight;
string buffer;
string failed_files;
gboolean failed = FALSE;
gboolean fd_ok = FALSE;
Date dt(ff->rh);
list<string>::iterator i;
	
//list<string> chosenlist; 

cout << "Fixfiles::do_fixfiles 1 ff->fixdate= " << ff->fixdate << endl;

// Traverse the pathname list and either TOUCH or FIXDATE each one

//if (ff->fr.get_request_pathname_multiple(chosenlist))
//  {

// It appears that the list is empty!
//    although the list is displayed
for (i=ff->pathname_list.begin(); i!=ff->pathname_list.end(); ++i)
	{
	fd_ok = TRUE;

cout << "Fixfiles::do_fixfiles 1a - pathname= " << *i << endl;
		
	// FIXDATE
	if (ff->fixdate)                 // Get the date from the file's header
		{
cout << "Fixfiles::do_fixfiles 2" << endl;

		type = ff->rh->get_file_type((*i), FALSE);
        
cout << "Fixfiles::do_fixfiles 3 (*i)= " << *i << endl;


		if (ff->rh->is_logged((*i), type))
			{
        
cout << "Fixfiles::do_fixfiles 4" << endl;

			UNLESS (dt.get_date(&ds, &tz, &daylight, (*i), type))
				{
				cout << "Fixfiles::do_fixfiles 5" << endl;
            
				buffer = "FIXDATE \"";
				buffer += (*i);
				buffer += "\" FAILED : Can't get header date";
				ff->rh->logfile->report(buffer);
				failed = TRUE;
				fd_ok = FALSE;
				failed_files += "    " + (*i) + '\n';
				continue;
				}
        // NOTE: GetDate returns UTC in ds
			}
		else
			{
cout << "Fixfiles::do_fixfiles 6" << endl;

			failed = TRUE;
			fd_ok = FALSE;
			buffer = "FIXDATE \"" + (*i) + "\" : NO HEADER - SKIPPED";
			ff->rh->logfile->report(buffer);
			failed_files += buffer + "\n";        
			}  
		}

	// TOUCH
	else                              // Get the current date
		{
		time (&ds); // gets UTC time, no timezone...
		}
  
    // Now do it...  
	if (fd_ok)
		{
		if (dt.date_file((*i), &ds)) // we don't need tz
			{
			if (ff->fixdate) buffer = "FIXDATE \"";
			else buffer = "TOUCH \"";
			buffer += (*i); //(char *)node->data;
			buffer += "\" ";
			while (buffer.length() < 25) buffer += ' ';
			buffer += " : OK";
			ff->rh->logfile->report(buffer);
			}
		else
			{
			cerr << "COULDN'T TOUCH/FIXDATE file \"" << (*i) << "\" : " << strerror(errno) << endl;
			if (ff->fixdate) buffer = "FIXDATE \"";
			else buffer = "TOUCH \"";
			buffer += (*i);
			buffer += "\" FAILED : ";
			buffer += strerror(errno);
			ff->rh->logfile->report(buffer);
			failed = TRUE;
			failed_files += "    " + (*i) + '\n';
			}
		}
    // else error already reported
	}
 
if (failed)
	{
	Complain *cn = new Complain(ff->rh->icon_pixbuf, &lightgrey, &pink, GTK_STOCK_CANCEL, GTK_STOCK_OK);

	if (ff->fixdate) buffer = ff->rh->quit->get_errTXT(FIXDATE_WARN);
	else  buffer = ff->rh->quit->get_errTXT(TOUCH_WARN);

	// NOTE: this carries on i.e. it isn't serial!
	buffer += '\n' + failed_files;
    
	cn->complain(CN_CONTINUE,
                (char *)((ff->fixdate) ? "FIXDATE" : "TOUCH"),
                (char *)buffer.c_str(),
                callback_continue, NULL, ff->rh);
	return;
	}

ff->rh->FrontPanel();
}











void Fixfiles::callback_delete_file(GtkWidget *widget, Fixfiles *ff)
{
GtkTreeIter iter;
GtkTreeSelection *select;
GtkTreeModel *model;

model = gtk_tree_view_get_model(GTK_TREE_VIEW(ff->treeview));
select = gtk_tree_view_get_selection (GTK_TREE_VIEW(ff->treeview));
gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

if ( (gtk_tree_selection_get_selected(select, &model, &iter)) )
  {
  gchar *str;
  gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
           0, // the column #,
           &str,
           -1);    


  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
  g_free(str);
  }
}






void Fixfiles::add_new_file(string pathname, Fixfiles *ff)
{
GtkTreeModel* model;
gboolean valid;
GtkTreeIter iter;
GtkTreeIter sibling;
gchar *str;
string name;

cout << "Fixfiles::add_new_file - 1: got " << pathname << endl;

if (!pathname.length()) return;

model = gtk_tree_view_get_model(GTK_TREE_VIEW(ff->treeview));

valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);


while (valid)
  {
  gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                    0, // the column #,
                    &str,
                    -1);
  name = str;
  g_free(str);

  if (pathname == name)
    {
    return;  // Don't add  if already present
    }
  else if (strcasecmp(pathname.c_str(), name.c_str()) < 0)
    {
    // INSERT IT HERE
    gtk_list_store_insert_before(GTK_LIST_STORE(model),
               &sibling,
               &iter); 
    gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0, pathname.c_str(), -1);
    ff->pathname_list.push_back(pathname);


cout << "Fixfiles::add_new_file - 2: done ff->pathname_list.push_back(" << pathname << ")" << endl;
    break;
    }
  
  valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
  }

if (!valid)
  {
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set (GTK_LIST_STORE(model), &iter, 0, pathname.c_str(), -1);
	ff->pathname_list.push_back(pathname);

cout << "Fixfiles::add_new_file - 3: done ff->pathname_list.push_back(" << pathname << ")" << endl;
  }

}






void Fixfiles::add_columns (GtkTreeView *treeview, gchar *title)
{
GtkCellRenderer *renderer;
GtkTreeViewColumn *column;

renderer = gtk_cell_renderer_text_new ();
column = gtk_tree_view_column_new_with_attributes(title,
                 renderer,
                 "text",
                 0,
                 NULL);
gtk_tree_view_column_set_sort_column_id (column, 0);
gtk_tree_view_append_column(treeview, column);

}



GtkTreeModel *Fixfiles::create_model(void)
{
GtkListStore *store;
GtkTreeIter iter;
list<string>::iterator i;

store = gtk_list_store_new (1, G_TYPE_STRING);
/*
if (mlist.empty())
  {
  // Add the project directory
  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, (gchar *)rh->project_path.c_str(), -1);  
  }
else for (i=mlist.begin(); i!=mlist.end(); ++i)
  {
  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, (gchar *)i->c_str(), -1);
  }
*/
return GTK_TREE_MODEL(store);
}






GtkWidget *Fixfiles::get_list_store()
{
GtkWidget *sw;
GtkTreeModel *model;


sw = gtk_scrolled_window_new (NULL, NULL);
gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
             GTK_SHADOW_ETCHED_IN);
gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
              GTK_POLICY_AUTOMATIC,
              GTK_POLICY_AUTOMATIC);

model = create_model(); //(mlist);

treeview = gtk_tree_view_new_with_model(model);
gtk_tree_view_set_rules_hint(GTK_TREE_VIEW (treeview), TRUE);
gtk_tree_view_set_search_column(GTK_TREE_VIEW (treeview),
               0);

g_object_unref (model);

gtk_container_add (GTK_CONTAINER (sw), treeview);

/* add columns to the tree view */
add_columns(GTK_TREE_VIEW (treeview), (gchar *)"SELECTED FILES");

gtk_widget_show_all(sw);
return sw;
}


void Fixfiles::callback_selection_changed(GtkWidget *widget, Fixfiles *ff)
{
//cout << "Fixfiles::callback_selection_changed" << endl;
gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(ff->file_chooser));

if (!filename)
	{
//	cout << "Fixfiles::callback_selection_changed - not a file" << endl;
	return;
	}

	
cout << "Fixfiles::callback_selection_changed - got" << filename << endl;
if (isfile((char *)filename))
		{
		ff->add_new_file( (string)filename, ff);		
		}
else cout << " : NOT A FILE" << endl;

g_free(filename);
}

/********************************************
 * Either TOUCH with the current datestamp  *
 * or FIXDATE by touching with the header's *
 * datestamp                                *
 ********************************************/

void Fixfiles::create_display(gboolean flag)
{
GtkWidget *grid1, *grid2, *paned, *frame1, *frame2; //, *vbox1, *vbox2;
GtkWidget *widget;
GtkWidget *vbox;
GtkWidget *hbox;
Date dt(rh);
string buffer;
fixdate = flag;

if (fixdate) dt.date_log_operation(&FixDateTXT);
else dt.date_log_operation(&TouchTXT );

rh->DestroyWidgets();

gtk_label_set_text (GTK_LABEL (rh->title), (fixdate) ? (char *)FixDateTXT.c_str() : (char *)TouchTXT.c_str() );

if (fixdate)
  {
  buffer = FixDateTXT + "\n";
  buffer += "  Make the modification datestamp of\n";
  buffer += "  selected files agree with the header datestamps.\n";
  buffer += "  The files then appear to be unmodified.\n";
  buffer += "  User informed if not possible, e.g. no header.\n";
  }
else
  {
  buffer = TouchTXT + "\n";
  buffer += "  Touch the selected files.\n";
  buffer += "  The modification datestamp of the selected files\n";
  buffer += "  is updated so the files appear modified.\n";
  }

buffer += "LEFT PANE\n  Select file(s) to be acted upon\n";
buffer += "RIGHT PANE\n  Selected Files: The files to be acted upon.";

gtk_widget_set_tooltip_text (GTK_WIDGET(rh->vbox), buffer.c_str() );

paned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
gtk_container_set_border_width(GTK_CONTAINER(paned), 10);

frame1 = gtk_frame_new (NULL);
frame2 = gtk_frame_new (NULL);

gtk_paned_pack1 (GTK_PANED (paned), frame1, TRUE, FALSE);
gtk_paned_pack2 (GTK_PANED (paned), frame2, TRUE, FALSE);
gtk_box_pack_start (GTK_BOX(rh->vbox), paned, TRUE, TRUE, 0);
	
gtk_paned_set_position(GTK_PANED(paned), rh->width/2);
  
vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
gtk_container_add(GTK_CONTAINER(frame1), vbox);



file_chooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_OPEN);
g_signal_connect(G_OBJECT(file_chooser), "selection-changed", G_CALLBACK(callback_selection_changed), this);
gtk_box_pack_start (GTK_BOX(vbox), file_chooser, TRUE, TRUE, 0);

  
vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
gtk_container_add(GTK_CONTAINER(frame2), vbox);
gtk_box_pack_start (GTK_BOX(vbox), get_list_store(), TRUE, TRUE, 0);


widget = gtk_button_new_with_label("Delete Selected File");
decorate_widget(widget, &black, NULL, NULL, NULL, NULL);
g_signal_connect (G_OBJECT (widget), "clicked",
            G_CALLBACK(callback_delete_file), this);
gtk_container_add(GTK_CONTAINER(vbox), widget);


	

	
hbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_EDGE);                                                      
gtk_container_add(GTK_CONTAINER(GTK_BOX (rh->vbox)), hbox);
                             
// CANCEL
widget = gtk_button_new_with_label( RevHead::CancelTXT.c_str() );
decorate_widget(widget,&black, NULL, NULL, NULL, GTK_STOCK_CANCEL);
g_signal_connect (G_OBJECT (widget), "clicked",
                        G_CALLBACK(QuitFixfiles), this);
gtk_container_add(GTK_CONTAINER(GTK_BOX (hbox)), widget);

// OK, DO IT
widget = gtk_button_new_with_label( RevHead::OKTXT.c_str() );
decorate_widget(widget, &black, NULL, NULL, NULL, GTK_STOCK_OK);
g_signal_connect (G_OBJECT (widget), "clicked",
            G_CALLBACK(do_fixfiles), this);
gtk_box_pack_end (GTK_BOX(hbox), widget, TRUE, TRUE, 0);

gtk_widget_show_all(GTK_WIDGET(rh->vbox));
}


