/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       header.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       28-Mar-2009 22.15.46 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Routines for writing the header                                           *
* (Separated from elsewhere)                                                *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "header.h"

#include "date.h"
//#include "changelog.h"

using namespace std;



// e.g. ADA  "--| " or for keyword "--|"
// i.e. keyword always one char left of normal

// The very start of the header
const string Header::start_comment_start[] =
                            {
                            //"**",       // RH_version
                            "/*",       // C
                            "//**",     // alternative 1 CPP 
                            "----",     // ADA
                            "/*",       // PHP
                            "/*",       // JAVA
                            "##",       // PERL
                            "!---",     // FORTRAN       
                            "##",       // HASH
                            "///",      // alternative 2 CPP 
                            };

const string Header::start_comment_end[] =
                            {
                            //"*",       // RH_version
                            "*",       // C
                            "*",       // alternative CPP 
                            "-",       // ADA
                            "*",       // PHP
                            "*",       // JAVA
                            "#",       // PERL
                            "-",       // FORTRAN 
                            "#",       // HASH
                            "//",       // alternative 2 CPP 
                            };

const string Header::comment_separator[] =
                            {
                            //"*",       // RH_version
                            "*",       // C
                            "*",       // alternative 1 CPP
                            "-",       // ADA  (1 line CPP only)
                            "*",       // PHP
                            "*",       // JAVA
                            "#",       // PERL
                            "-",       // FORTRAN
                            "#",       // HASH
                            "/",       // alternative 2 CPP 
                            };


const string Header::in_comment_start[] =
                            {
                            //"* ",       // RH_version
                            "* ",       // C
                            "//* ",     // alternative 1 CPP 
                            "--| ",     // ADA
                            "* ",       // PHP
                            "* ",       // JAVA
                            "# ",       // PERL
                            "!-| ",     // FORTRAN
                            "# ",       // HASH
                            "// ",      // alternative 2 CPP 
                            };

const string Header::in_comment_key_start[] =
                            {
                            //"*",       // RH_version
                            "*",       // C
                            "//*",     // alternative 1 CPP 
                            "--|",     // ADA
                            "*",       // PHP
                            "*",       // JAVA
                            "#",       // PERL
                            "!-|",     // FORTRAN
                            "#",       // HASH
                            "//",      // alternative 2 CPP 
                            };



const string  Header::in_comment_end[] =
                            {
                            //"*",       // RH_version
                            "*",       // C
                            "*",       // alternative 1 CPP 
                            "|",       // ADA
                            "*",       // PHP
                            "*",       // JAVA
                            "#",       // PERL
                            "|",       // FORTRAN
                            "#",       // HASH
                            "//",       // alternative 2 CPP 
                            };



const string Header::end_comment_start[] =
                            {
                            //"**",       // RH_version
                            "**",       // C
                            "//**",     // alternative 1 CPP 
                            "----",     // ADA
                            "**",       // PHP
                            "**",       // JAVA
                            "##",       // PERL
                            "!---",     // FORTRAN
                            "##",       // HASH
                            "///",      // alternative 2 CPP 
                            };





// THIS is the LAST CHARACTER only (mostly)
const string Header::end_comment_end[] =
                            {
                            //"*",       // RH_version at EOL
                            "/",       // C at EOL
                            "*",       // alternative 1 CPP
                            "-",       // ADA
                            "/",       // PHP
                            "/",       // JAVA
                            "#",       // PERL
                            "-",       // FORTRAN
                            "#",       // HASH
                            "//",       // alternative 2 CPP 
                            };



string const Header::Project_DetailsTXT    = " Project Details";
string const Header::UnderLineTXT          = " ---------------";
string const Header::AuthorTXT             = "Author:";


Header::Header(RevHead *revhead)
{
cout << "Header:\n";
rh = revhead;
}

Header::~Header(void)
{
cout << "~Header:\n";
}
 


/****************************************
 * gboolean write_uf_string(fp, s type) *
 * Write an unformatted string to file. *
 * Assumes line ended with zero.        *
 * Return TRUE (OK) or FALSE.           *
 ****************************************/

gboolean Header::write_uf_string(fstream *ofp, string &s, int type)
{
string buffer = in_comment_start[type] + s;
return( write2EOL(ofp, buffer, type) );
}
 
 
/*********************************
 * gboolean WriteRHEnd(fp, type) *
 * Returns TRUE (OK) or FALSE.   *
 *********************************/

gboolean Header::write_RHEnd(fstream *ofp, int type)
{
string buffer = in_comment_key_start[type] + RevHead::RH_End;

write2EOL(ofp, buffer, type);

// Last line, of asterisks
return(write_end_separator(ofp, type));
}




gboolean Header::write2EOL(fstream *ofp, string &s, int type)
{
 
unsigned int length = LINE_LENGTH + in_comment_start[type].length() - 1;

while (s.length() < length) s += ' ';

s += in_comment_end[type] + "\n";


return( (ofp->write(s.c_str(), s.length())) ? TRUE : FALSE );
}



/********************************************
 * gboolean write_start_separator(fp, type) *
 * A separating row within the header       *
 * Returns TRUE (OK) or FALSE.              *
 ********************************************/

gboolean Header::write_start_separator(fstream *ofp, int type)
{
string buffer = start_comment_start[type];

unsigned int length = LINE_LENGTH + buffer.length() - 1;
  
while (buffer.length() < length)
  {
  buffer += comment_separator[type];
  }
buffer += start_comment_end[type] + "\n";

return( (ofp->write(buffer.c_str(), buffer.length())) ? TRUE : FALSE );  
}



/*****************************************
 * string get_in_separator(s, type)      *
 * A separating row within the header    *
 * Returns string in_separator.          *
 *****************************************/
string & Header::get_in_separator(string &s, int type)
{
s = Header::in_comment_key_start[type] + comment_separator[type];
unsigned int length = LINE_LENGTH + s.length() - 1;

while (s.length() < length)
  {
  s += comment_separator[type];
  }
s += in_comment_end[type] + "\n";
return s; 
}

gboolean Header::write_in_separator(fstream *ofp, int type)
{
string buffer;
get_in_separator(buffer, type);

return( (ofp->write(buffer.c_str(), buffer.length())) ? TRUE : FALSE );  
}


/******************************************
 * gboolean write_end_separator(fp, type) *
 * Terminal separating row of the header  *
 * Returns TRUE (OK) or FALSE.            *
 ******************************************/

gboolean Header::write_end_separator(fstream *ofp, int type)
{
string buffer = end_comment_start[type];
unsigned int length = LINE_LENGTH + buffer.length() - 1;

while (buffer.length() < length)
  {
  buffer += comment_separator[type];
  }
buffer += end_comment_end[type] + "\n";

return( (ofp->write(buffer.c_str(), buffer.length())) ? TRUE : FALSE );  
}


/************************************
 * gboolean write_spaces(fp, type)  *
 * Returns TRUE (OK) or FALSE.      *
 ************************************/

gboolean Header::write_spaces(fstream *ofp, int type)
{  
string buffer = in_comment_start[type]; 
unsigned int length = LINE_LENGTH + buffer.length() - 1;

while (buffer.length() < length)
  {
  buffer += ' ';
  }
buffer += in_comment_end[type] + "\n";
  
return( (ofp->write(buffer.c_str(), buffer.length())) ? TRUE : FALSE );
}


/****************************************
 * gboolean write_slist(fp, list, type) *
 * Returns TRUE (OK) or FALSE.          *
 ****************************************/

gboolean Header::write_slist(fstream *ofp, list<string> alist, int type)
{
list<string>::iterator i;  
for (i=alist.begin(); i!=alist.end(); ++i)
  {
  UNLESS (write_uf_string(ofp, (*i), type))
    {
    return(FALSE);
    }
  }
  
return(TRUE);
}


gboolean Header::write_file_info(fstream *ofp, class Files *f, int type )
{
string buffer;
string key_head = in_comment_key_start[type];
    
// spaces
if (write_spaces(ofp, type))
  {

// This File
  buffer = in_comment_key_start[type];
  buffer += "This File:       ";
//  buffer += *fname;
  buffer += f->name;
  if (write2EOL(ofp, buffer, type))
    {

// ===================================================
// File Version if wanted  WE NEED TO KNOW IF WANTED
// ===================================================

    if (f->flags.version_wanted)
      {

      buffer = key_head + RevHead::F_VersionTXT + "    ";
      rh->append_version(buffer, f->major_version, f->minor_version, f->micro_version);        
      if (!write2EOL(ofp, buffer, type))
        {
        return FALSE;
        }
      }


      
// Author
    buffer = key_head + AuthorTXT + "          ";
//    buffer +=  *fauthor;
    buffer +=  f->author;
    if (!f->author_email.empty())
      {
      buffer += " <";
      buffer += f->author_email;
      buffer += ">"; 
      }
    if (write2EOL(ofp, buffer, type))
      {
      Date dt(rh);
      string datebuffer;
// Datestamp
      buffer = key_head + Date::D_StampTXT + "       ";
      dt.ds2date(&f->datestamp, &f->timezone, &f->daylight, &datebuffer, D_M_Y_T);
      buffer += datebuffer;
      if (write2EOL(ofp, buffer, type) && write_spaces(ofp, type))
        {
          
// File Summary       OPTIONAL
        if (!f->summary.empty())
          {   
          buffer = key_head + RevHead::F_SummaryTXT;
          if ( ! (write2EOL(ofp, buffer, type) &&
                    write_slist(ofp, f->summary, type) &&
                    write_spaces(ofp, type)) )
            {
            cerr << "    FAILED to write file summary" << endl;
            return FALSE;
            }
          }
        }
      else
        {
        cerr << "    FAILED to write spaces" << endl;
        return FALSE;
        }

      if ( write_RHEnd(ofp, type) ) // includes final separator line
        {
        return TRUE;
        }
      cerr << "    FAILED to write RevHead_End" << endl;
      }
    }
  }
return(FALSE);
}




/************************************
 * gboolean write_extra_directories *
 * (to RH_ChangeLog header)         *
 * Returns TRUE (OK) or FALSE.      *
 ************************************/
gboolean Header::write_source_directories(fstream *ofp, int type)
{
if ( rh->source_path.empty() ) return TRUE;
  
if (write_uf_string(ofp, RevHead::SourceDirsTXT, type))
  {
  if (write_slist(ofp, rh->source_path, type))
    {
    return (write_spaces(ofp, type));
    }
  }
return FALSE;
}

/************************************
 * gboolean WritePDate(fp, type)    *
 * Returns TRUE (OK) or FALSE.      *
 ************************************/

gboolean Header::write_PDate(fstream *ofp, int type)
{
string buffer;

buffer = in_comment_key_start[type];
buffer += RevHead::P_DateTXT + "    ";
buffer +=rh->datebuffer;
return(write2EOL(ofp, buffer, type));
}


/************************************
 * gboolean WritePVersion(fp, type) *
 * Returns TRUE (OK) or FALSE.      *
 ************************************/

gboolean Header::write_PVersion(fstream *ofp, int type)
{
string buffer;

buffer = in_comment_key_start[type];
buffer += RevHead::P_VersionTXT + " ";

rh->append_version(buffer, rh->p_major_version, rh->p_minor_version, rh->p_micro_version);        

return( write2EOL(ofp, buffer, type) );
}

/************************************
 * gboolean Write_PName(fp, type)   *
 * Returns TRUE (OK) or FALSE.      *
 ************************************/

gboolean Header::write_PName(fstream *ofp, int type)

{
string buffer;

buffer = in_comment_key_start[type];
buffer += RevHead::P_NameTXT + "    ";
UNLESS ( rh->p_name.empty() ) buffer += rh->p_name;
return( write2EOL(ofp, buffer, type) );
}


/************************************
 * gboolean write_POwner(fp, type)  *
 * Returns TRUE (OK) or FALSE.      *
 ************************************/

gboolean Header::write_POwner(fstream *ofp, int type)
{
string buffer;

// NOTE: Assigning the buffer on one line results in "P_OwnerTXT instead of *P_OwnerTXT 
// This has to be a bug in the compiler

buffer = in_comment_key_start[type];
buffer += RevHead::P_OwnerTXT + ' ' + rh->p_owner;
  
if (!rh->p_owner_email.empty()) buffer += " <" + rh->p_owner_email + '>'; 
return( write2EOL(ofp, buffer, type) );
}



/******************************************
 * gboolean write_Project(fh, type, opts) *
 * Write the project details portion      *
 * of a header.                           *
 * Returns TRUE (OK) or FALSE (failed).   *
 ******************************************/

gboolean Header::write_Project(fstream *ofp, int type, string &options, bool changelog)
{
string buffer;

//cout << "Header::write_Project 1" << endl;
  
// start separator

if (write_start_separator(ofp, type))
  {
//cout << "Header::write_Project 2" << endl;

// RevHead_Start
    
  buffer = in_comment_key_start[type];
  buffer += RevHead::RH_Start;
  buffer += options;
  if (write2EOL(ofp, buffer, type))
    {
// spaces
    if (write_spaces(ofp, type))
      {
//cout << "Header::write_Project 3" << endl;

// Project Details    
      buffer = in_comment_key_start[type];
      buffer += Project_DetailsTXT;
      if (write2EOL(ofp, buffer, type))
        {
//cout << "Header::write_Project 4" << endl;
// underline
        buffer = in_comment_key_start[type];
        buffer += UnderLineTXT;
        if (write2EOL(ofp, buffer, type))
          {
// Project Name                      REQUIRED
          if (write_PName(ofp, type))
            {
// Project Version                   REQUIRED 
            if (write_PVersion(ofp, type))
              {

// Copyright owner and email address REQUIRED
              if (write_POwner(ofp, type))
                {
// Copyright                         OPTIONAL

                if ( !rh->p_copyright.empty() )
                  {
                  buffer = in_comment_key_start[type] + RevHead::P_CopyrightTXT;
                  if ( ! ( write2EOL(ofp, buffer, type) &&
                          write_slist(ofp, rh->p_copyright, type) &&
                          write_spaces(ofp, type) ) )
                    {
                    return FALSE;
                    }
                  }
// Date
                if (write_PDate(ofp, type) && write_spaces(ofp, type))

// Date started
                  {
                  Date dt(rh);
                  string datebuffer;
                  buffer = in_comment_key_start[type];
                  buffer += RevHead::Start_DSTXT + " ";
                                
                  dt.ds2date(&rh->p_startds, &rh->p_starttz, &rh->p_start_daylight, &datebuffer, D_M_Y_T);
                
                  buffer += datebuffer;
                
                  if (write2EOL(ofp, buffer, type))
                    {
                    if (write_spaces(ofp, type))
                      {
// Contact                OPTIONAL
                      if (!rh->p_address.empty())
                        {
                        buffer = in_comment_key_start[type] + RevHead::ContactTXT;

                        if ( !( write2EOL(ofp, buffer, type) &&
                                write_slist(ofp, rh->p_address, type) &&
                                write_spaces(ofp, type) ) )
                          {
                          return FALSE;
                          }
                        }
                                    
// Project Summary                   OPTIONAL
//cout << "Header::write_Project 5 - SUMMARY" << endl;
                      if ( (!changelog) && (!rh->p_summary.empty()) )
                        {
                        buffer=Header::in_comment_key_start[type] + RevHead::P_SummaryTXT;
                        if  ( !( write2EOL(ofp, buffer, type) &&
                                  write_slist(ofp, rh->p_summary, type) &&
                                  write_spaces(ofp, type) ) )
                          {
                          // FAILED
                          return FALSE;
                          }
                        }
                  
// Extra paths to process OPTIONAL
                      if (changelog && !rh->source_path.empty())
                        {
                        buffer=Header::in_comment_key_start[type] + RevHead::SourceDirsTXT;
                    
                        if  ( !( write2EOL(ofp, buffer, type) &&
                                  write_slist(ofp, rh->source_path, type) &&
                                  write_spaces(ofp, type) ) )
                          {
                          // FAILED
                          return FALSE;
                          }                
                        }

// ChangeLog path         OPTIONAL
                      if (changelog && !rh->changelog_path.empty())
                        {
                        buffer=Header::in_comment_key_start[type] + RevHead::ChangeLogDirTXT;
                        if  ( !( write2EOL(ofp, buffer, type) &&
                                  write_uf_string(ofp, rh->changelog_path, type) &&
                                  write_spaces(ofp, type) ) )
                          {
                          // FAILED
                          return FALSE;
                          }                
                        }

//cout << "Header::write_Project 6" << endl;

// End with asterisks
                      if (write_in_separator(ofp, type))
                        {
//cout << "Header::write_Project 7" << endl;
                        return(TRUE);
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
return(FALSE);
}

gboolean Header::write_Project(fstream *ofp, int type, string &options)
{
write_Project(ofp, type, options, FALSE); 
}
