/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       misc.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Apr-2013 10.22.12 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* General purpose functions                                                 *
* 20-Aug-2004 12.10.29 -36000 0.00    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <string.h>
#include <linux/limits.h>

#include "misc.h"

#include "logfile.h"

using namespace std;


namespace misc
{
//string P_OwnerTXT = "Copyright_Owner";
  
// "lstat" MUST be used here because a link
// returns the statbuf of its target with "stat"

/*********************************************
 * Return TRUE if pathname is a regular file *
 *********************************************/

gboolean isfile(const char *pathname)
{
  struct stat64 statbuf;

  if (!(lstat64(pathname, &statbuf)))
    {
    if (S_ISREG(statbuf.st_mode)) return(TRUE);
    }

  return(FALSE); // Not a regular file or ERROR
}

/******************************************
 * Return TRUE if pathname is a directory *
 ******************************************/
gboolean isdir(const char *pathname)
{
struct stat64 statbuf;
if (!(lstat64(pathname, &statbuf)))
  {
  if (S_ISDIR(statbuf.st_mode)) return(TRUE);
  }
return(FALSE);
}


/******************************************
 * Return TRUE if fname is a regular file *
 * AND fname is readable.
 ******************************************/
gboolean isreadable_file(const char *pathname)
{
if (isfile(pathname))
  {
  if (!access(pathname, R_OK))
    {
    return(TRUE);
    }
  }
return(FALSE); // Not a regular file or not readable or ERROR
}


  
void get_filelist(char *directory, gboolean fileflag, list<string> &alist, RevHead *rh)
{
DIR *dp;
struct dirent *de;
char *fname ;
char buffer[PATH_MAX];

strcpy(buffer, directory);
if (strlen(buffer) > 1) strcat(buffer, "/");
fname = &buffer[strlen(buffer)]; // address of terminal zero
 
if ( (dp = opendir(buffer)) )
    {
    while ( (de = readdir(dp)) )
        {
        strcpy(fname, de->d_name);
        
        if ( ((fileflag) && (isfile(buffer))) || ((!fileflag) && isdir(buffer)) )
            {
            // We have a regular file or directory so add it to our list in alphabetical order
            //alist.insert_sorted(fname);
            alist.push_back(fname);
            alist.sort();
            alist.unique();
            }
        }
    closedir(dp);
    }
else
    {
    string s = "Can't open directory: \"";
    s += buffer;
    s += '\"';
    rh->logfile->report(s);
    }
return;
}
  

bool isseparator(char x)
{
return ((x == ' ')||(x == '\t')||(x == '=')||(x == ':'));
}

// This should allow quoted strings...
void getright(string &s, string &v)
{
unsigned int p=0;
bool inquote = FALSE;

v = "";  // start with an empty string!

while ( (p < s.length()) && isblank(s[p]) )
  {
  p++;
  }

if ( (p < s.length()) && ((s[p]=='=') || (s[p]==':')) )
  {
  p++;
  while ( (p < s.length()) && isblank(s[p]) )
    {
    p++;
    }
  }

while (p < s.length())
  {
  if (s[p] == '\"')
    {
    inquote = !inquote;
    p++;
    continue;
    }

  // If any of these the rest of the line is ignored unless in quote
  // Horizontal white space has to be allowed
  if ( (!inquote) &&
//    (isblank(s[p]) || (s[p]=='#') || (s[p]==';') || (s[p]=='\n')) ) break;
    ((s[p]=='#') || (s[p]==';') || (s[p]=='\n')) ) break;

    v += s[p];
    p++;
  }
}

bool getline(fstream &ifp, string &line, bool clear_leadin)
{
char c;

if (ifp.eof()) return FALSE;

line = "";
while ( !ifp.eof() )
  {
  ifp.get(c);
  if ((c == 0x0a) || (c == 0) || (ifp.eof() )) break;
  line += c;
  }
if (clear_leadin) remove_leading(line);
remove_trailing(line);
return TRUE;
}

bool getvalue(string &line, string &name, string &value)
{
if ( STRIEQN(name.c_str(), line.c_str(), name.length()) &&
    isseparator(line[name.length()]) )
  {
  string s;
  string r;
  s = line.substr(name.length());
  getright(s, r);
  remove_leading(r);  
  remove_trailing(r);
  value = r;
  return TRUE;
  }
return FALSE;
}

bool getvalue(fstream &ifp, string &line, string &startname, string &endname, list<string> &value)
{
if ( STRIEQ(startname.c_str(), line.c_str()) )
  {
  while (getline(ifp, line, FALSE))
    {
    if (startname != Start_ChangelogTXT) remove_leading(line); // changelog keeps leading blanks
    if ( STRIEQ(endname.c_str(), line.c_str()) )
      {
      return TRUE;
      }
    if (line.empty()) line = "\n";  // Allow for empty lines esp. in private copy of changelog
    value.push_back(line);
    }
  }
return FALSE;
}

// Create a button widget  
GtkWidget *CreateButton(GtkWidget *box, const char *string)
{
GtkWidget *button;

/* --- Get the check button --- */
button = gtk_button_new_with_label (string);

/* --- Pack the checkbox into the vertical box (box). --- */
gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 10);

/* --- Show the widget --- */
gtk_widget_show(button);

return (button);
}


/***************************************
 * Colourize a widget                  *
 * fg bg prelight stock_id             *
 * Do nothing for NULL colour pointers *
 ***************************************/
void decorate_widget(GtkWidget *widget, GdkRGBA *fg_normal, GdkRGBA *bg_normal, GdkRGBA *fg_prelight,  GdkRGBA *bg_prelight, const gchar *stock_id)
{
if (fg_normal) gtk_widget_override_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, fg_normal);
	
// Background doesn't seem to work as it should: it works with ghosted (insensitive) widgets instead of normal
if (bg_normal) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, bg_normal);

// This is actually the prelight fg, not bg
if (fg_prelight) gtk_widget_override_color(GTK_WIDGET(widget), GTK_STATE_FLAG_PRELIGHT, fg_prelight);

if (bg_prelight) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_PRELIGHT, bg_prelight);

if (GTK_IS_BUTTON(widget) && (stock_id) && *(stock_id))
  {
  gtk_button_set_image(GTK_BUTTON(widget), gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON));
  // Make sure the image is displayed!
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(widget)))); 
  }
}


/****************************************
 * CreateCheckBox                       *
 * Add a checkbox to the packing widget *
 ****************************************/
GtkWidget *CreateCheckBox (GtkWidget *box, char *label)
{
GtkWidget *check;

check = gtk_check_button_new_with_label (label);
gtk_box_pack_start (GTK_BOX (box), check, FALSE, FALSE, 10);
gtk_widget_show (check);
return (check);
}

// ====================================================
// CreateCombobox
//
// Create a drop down combobox with a few items in it.
//
// ====================================================

void SetComboTextList(GtkWidget *combo, GList *list)
{
  GList *text;

  for (text=list; text; text=text->next)
    {
//      gtk_combo_box_append_text(GTK_COMBO_BOX(combo), (gchar *)text->data);

		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), (gchar *)text->data);
    }

  gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);  // Make visible

}

GtkWidget *CreateCombobox (GList *list)
{
  GtkWidget *combo;

  // --- Make a combo box.

  // USE: GtkWidget*  gtk_combo_box_new_text(void);
  // A straight combo is deprecated...
  // This will mean that a lot of the code will have to be re-written
  // Use the following to manipulate the combobox
  // gtk_combo_box_append_text()
  // gtk_combo_box_insert_text()
  // gtk_combo_box_prepend_text()
  // gtk_combo_box_remove_text()
  // It accepts the "changed" signal same as combo


//  combo = gtk_combo_box_new_text();
  combo = gtk_combo_box_text_new(); 

  SetComboTextList(combo, list);

  // --- Make it visible
  gtk_widget_show(combo);
  
  return (combo);
}

/****************************************
 * GtkWidget *add_page(notebook, title) *
 * Add page to notebook                 *
 ****************************************/
GtkWidget *add_page(GtkWidget *notebook, gchar *title)
{
GtkWidget *label;
GtkWidget *vbox;

vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
gtk_box_set_homogeneous (GTK_BOX (vbox), FALSE);


	
gtk_widget_show (vbox);

label = gtk_label_new(title);
gtk_notebook_append_page(GTK_NOTEBOOK (notebook), vbox, label);
return(vbox);
}

/********************************
 * Clear out the existing list. *
 ********************************/

void free_list(GList *list)
{
  GList *node;
 
  for (node = list; node; node = node->next)
    {
    if (node->data) free(node->data);
    }
  g_list_free(list);
  return;
}


void remove_leading(string &s)
{
while (s.length() > 0)
    {
    if ( (s[0] == ' ') || (s[0] == '\t') )
        {
        s = s.substr(1);
        }
    else break;
    }
}

void remove_trailing(string &s)
{
if ( (s.length() > 0) && (s[s.length()-1] == '\n') )
  {
  s.resize(s.length()-1);
  }
while (s.length() > 0)
  {
  if ( (s[s.length()-1] == ' ') || (s[s.length()-1]== '\t') )
    {
    s.resize(s.length()-1);
    }
  else break;
  }

}

// NOTE: remove_head should be used BEFORE remove_tail
void remove_tail(string &s, string &tail)
{
if (s.length())
  {
  unsigned int len  = s.length();  // index of last char
  unsigned int tlen = tail.length();

  if ( (len >= 0) && (s.substr(len-1, 1) == "\n") )
    {
    s.erase(--len);
    }

if ( (len >= tlen) && (s.substr(len-tlen, tlen) == tail) )
    {
    while (tlen) // remove the tail characters
      {
      s.erase(--len);
      tlen--;
      }
    }
   
  while (len > 0) // now remove the trailing blanks
    {
    if ( (s.substr(len-1, 1) == " ") || (s.substr(len, 1) =="\t") )
        {
        s.erase(--len);
        }
    else break;
    }
  }
}


void remove_head_and_tail(string &s, string &head,string &tail)
{  
if (s.length())
  {
  if (s.length() >= head.length())
    {    
    if (s.substr(0, head.length()) == head)
      {
      s = s.substr(head.length());
      }                                      
    }
  while ((s[0] == ' ') || (s[0] == '\t')) s = s.substr(1);
  }
  
remove_tail(s, tail);
}

// s_toupper and s_tolower are overloaded
string s_toupper(string s)
{
misc::case_str.clear();
for (unsigned int n = 0; n<s.length(); n++)
    misc::case_str += toupper(s[n]);
return misc::case_str;
}

string s_tolower(string s)
{
misc::case_str.clear();
for (unsigned int n = 0; n<s.length(); n++)
    misc::case_str += tolower(s[n]);
return misc::case_str; 
}

void s_toupper(string *s)
{
for (unsigned int n = 0; n<s->length(); n++)
    (*s)[n] = toupper((*s)[n]);
return;
}

void s_tolower(string *s)
{
for (unsigned int n = 0; n<s->length(); n++)
    (*s)[n] = tolower((*s)[n]);
return;
}





  /*******************************************
   * int suficmp(char * string, char *suffix *
   * Compare the suffix "suffix" with the    *
   * terminal part of "string" and return as *
   * a normal case insensitive compare.      *
   *******************************************/
  
  int suficmp(const char *string, const char *suffix)
  {
    int rc;
    
    int len_suffix = strlen(suffix);
    int len_string = strlen(string);
    
    if ( (rc = len_string - len_suffix) >= 0 )
      {
        rc = strcasecmp(&string[rc], suffix);
      }
    return(rc);
  }

  /*******************************************
   * int sufcmp(char * string, char *suffix  *
   * Compare the suffix "suffix" with the    *
   * terminal part of "string" and return as *
   * a normal case-sensitive compare.        *
   *******************************************/
  
int sufcmp(const char *string, const char *suffix)
{
int rc;

int len_suffix = strlen(suffix);
int len_string = strlen(string);

if ( (rc = len_string - len_suffix) >= 0 )
    {
    rc = strcmp(&string[rc], suffix);
    }
return(rc);
}


  // I don't seem to be able to make the following functions "inline"
  int STREQ(const char *x, const char *y)
  {
    return(strcmp(x,y)==0);
  }

  int STRIEQ(const char *x, const char *y)
  {
    return(strcasecmp(x,y)==0);
  }
  int STREQN(const char *x, const char *y, int n)
  {
    return(strncmp(x,y,n)==0);
  }
  int STRIEQN(const char *x, const char *y, int n)
  {
    return(strncasecmp(x,y,n)==0);
  }
  int SUFEQ(const char *x, const char *y)
  {
    return(sufcmp(x,y)==0);
  }
  int SUFIEQ(const char *x, const char *y)
  {
    return(suficmp(x,y)==0);
  }

}
