/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       quit.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       24-Mar-2010 20.16.29 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* A re-write of the quit routines split off from RevHead.c.  Added Usage()  *
* to cover CLI start.                                                       *
* 20-Dec-1995 22.03.23.14  0.00    DMB                                      *
* 24-Dec-1995 13.34.57.01  0.01    DMB                                      *
* Fixed Usage() to display how RevHead is used from SHELL and Workbench.    *
* Now displayed as long as user wants with the CANCEL gadget when done.     *
* 24-Dec-1995 13.34.57 0  0.01    DMB                                       *
* Error management and exit routines                                        *
* 20-Aug-2004 12.10.29 -36000 0.6     DMB                                   *
* Very similar to previous but reflects C++                                 *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/


#include "quit.h"


#include "logfile.h"
#include "date.h"
#include "tfile.h"
#include "misc.h"
#include "complain.h"


using namespace std;
using namespace misc;


// Doing it this way gets rid of the errors
// It seems that static data MUST ALWAYS be initialized this way...
int Quit::rc = EXIT_SUCCESS;

int Quit::n1 = 0;
char *Quit::str1 = NULL;
char *Quit::str2 = NULL;
RevHead *Quit::rh = NULL;


Quit::Quit(void)
{
cout << "Quit:\n";
rh = NULL;
}

Quit::~Quit(void)
{      
cout << "~Quit:\n";
}

void Quit::set_revhead(RevHead *revhead)
{
rh = revhead;
}

// Get return code
int Quit::get_rc(void)
{
return(rc);
}

/****************************************
 * cleanexit()                          *
 * Close everything and clean up memory	*
 ****************************************/
void Quit::cleanexit(int errnum)
{
string buffer;
Date dt(rh);
TFile tf;
int n = rh->logfile->changed;


//UNLESS ( rh->tch_name.empty() ) 
//    {
//    buffer = "REMOVING " + rh->tch_name;
//    rh->logfile->write2log(buffer);
//    remove( (char *)rh->tch_name.c_str() );
//    rh->tch_name.clear();
//    }

  
buffer = "Quit::cleanexit(" + tf.itoa(errnum);
buffer += ')';
dt.date_log_operation(&buffer);

UNLESS (n)
  {
  // Delete the unchanged log if in overwrite mode, otherwise keep it
  if (!rh->flags.append_log) rh->logfile->delete_logfile();
  }

rh->logfile->close();
delete rh->logfile;
rh->logfile = NULL;



//rh->default_file.clear();
rh->np_owner.clear();
rh->np_major_version.clear();
rh->np_minor_version.clear();
rh->np_micro_version.clear();
rh->nf_author.clear();

rh->p_name.clear();
rh->p_owner.clear();
rh->p_major_version.clear();
rh->p_minor_version.clear();
rh->p_micro_version.clear();
rh->path.clear();
rh->progname.clear();

rh->file_list.clear();

n = errnum;
if (n <= RH_WARNING) n = EXIT_SUCCESS;
else n = EXIT_FAILURE;
rc = n; 
  

this->~Quit();

cout << "Leaving Quit::cleanexit" << endl;
if (n == EXIT_SUCCESS) cout << "EXIT_SUCCESS" << endl;
else cout << "EXIT_FAILURE" << endl;


exit(n);  

}



char *Quit::get_errTXT(int errnum)
{
static const char *ErrorTXT[ NUMBER_OF_ERRORS] =
 {

/* ALL OK */
    "",                                             // NO_ERR               0
    "<*** USER ABORT ***>",                         // USER_ABORT_ERR       1


/* WARNINGS */
    "Can't create/change icon for \"%s\"",          // CREATE_ICON_WARN     2
    "Can't open log file \"%s\"",                   // NO_OPEN_WARN         3
    "Can't delete \"%s\"",                          // NO_DELETE_WARN       4
    "Can't TOUCH file(s)",                          // TOUCH_WARN           5
    "Can't FIXDATE file(s)",                        // FIXDATE_WARN         6

/* ERRORS */
    "Can't fix datestamp \"%s\"",                   // DS_ERROR             7
    "Version file not written,\"%s\"",              // VERSION_ERROR        8
    "Can't write filenote to \"%s\"",               // FILENOTE_ERROR       9
    "Can't copy filenote from \"%s\" to \"%s\"",    // COPY_FILENOTE_ERROR  10

/* FATAL ERRORS */
    "Can't obtain current time datestamp",          // TIME_F_ERR           11
    "Out of memory, can't allocate \"%s\"",         // MEMORY_F_ERR         12
    "Can't open window",                            // WINDOW_F_ERR         13
    "Workbench error, no arglist",                  // WORKBENCH_F_ERR      14
    "Can't get icon diskobject \"%s\"",             // ICON_F_ERR           15
    "Arg defined more than once \"%s\"",            // ARG_TWICE_F_ERR      16
    "Missing arg \"%s\"",                           // MISSING_ARG_F_ERR    17
    "Can't chdir() to PATH \"%s\"",                 // PATHLOCK_F_ERR       18
    "Can't get REVHEAD program's path",             // PROGPATH_F_ERR       19
    "Can't open temporary file for copy of %s",     // TEMP_OPEN_F_ERR      20
    "Can't get current path",                       // PATH_F_ERR           21
    "Can't rename file \"%s\" as \"%s\"",           // RENAME_F_ERR         22
    "Can't open \"%s\"",                            // OPEN_F_ERR           23
    "Can't read \"%s\"",                            // READ_F_ERR           24
    "Can't write \"%s\"",                           // WRITE_F_ERR          25
    "Can't understand \"%s\"",                      // UNDERSTAND_F_ERR     26
    "TOO MANY ERRORS!",                             // TOO_MANY_ERR         27
    "Can't opendir() current directory \"%s\"",     // EXAMINE_F_ERR        28
    "Can't readdir() current directory \"%s\"",     // READDIR_F_ERR        29
    "Can't closedir() current directory \"%s\"",    // CLOSEDIR_F_ERR       30
    "Can't access() \"%s\"",                        // ACCESS_F_ERR         31
    "Can't update project header details, \"%s\"",  // PROJ_UPDATE_F_ERR    32
    "Can't seek file \"%s\"",                       // SEEK_F_ERR           33
    "Can't log new file \"%s\"",                    // LOG_F_ERR            34
    "Can't copy file \"%s\" to \"%s\"",             // COPY_F_ERR           35
    "Can't read \"%s\" and/or write \"%s\"",        // READWRITE_F_ERR      36
    "Can't create file \"%s\"",                     // CREATEFILE_F_ERR     37
    "Can't update file \"%s\"",                     // FILE_UPDATE_F_ERR    38
    "Can't write file changes to \"%s\"",           // FILECHANGES_F_ERR    39
    "<*** UNKNOWN ERROR ***>",                      // UNKNOWN_ERR          40
  };

if (errnum > UNKNOWN_ERR || errnum < 0) errnum = UNKNOWN_ERR;
return((char *)ErrorTXT[errnum]);    
}





/****************************************
 * HandleError()                        *
 * Send the zero terminated error string*
 * to both the error file if present    *
 * AND to stderr.                       *
 * This should ensure that the user is	*
 * informed!                            *
 ****************************************/
void Quit::HandleError(int errnum, const char *fname1, const char *fname2)
{
string buffer;
TFile tf;
char *c;

if (errnum > UNKNOWN_ERR || errnum < 0) errnum = UNKNOWN_ERR;

if (errnum==1)
  {
  buffer = "#" + tf.itoa(errnum);
  buffer += ": ";
  }
else if (errnum && (errnum <= RH_WARNING))
  {
  buffer = "WARNING #" + tf.itoa(errnum);
  buffer += ": ";
  }
else if (errnum >= RH_FATAL_ERROR)
    {
    buffer = "FATAL ERROR #" + tf.itoa(errnum);
    buffer += ": ";
    }
else if (errnum > RH_WARNING)
    {
    buffer = "ERROR #" + tf.itoa(errnum);
    buffer += ": ";
    }

// NOTE: fname1 and fname2 may be NULL

c = get_errTXT(errnum);

while (*c && (*c != '%'))
    {
    buffer += *c;
    c++;
    }
if (*c == '%')
    {
    c++;
    if (*c == 's')
        {
        c++;
        if (fname1)
            {
            buffer += fname1;
            }
        }
    }
while (*c && (*c != '%'))
    {
    buffer += *c;
    c++;
    }

if (*c == '%')
    {
    c++;
    if (*c == 's')
        {
        c++;
        if (fname2)
            {
            buffer += fname2;
            }
        }
    }
while (*c)
    {
    buffer += *c;
    c++;
    }

rh->logfile->write2log(buffer);

if (errnum > RH_WARNING) rh->numerrs++;

if (rh->numerrs > MAX_ERRORS)
	{
  rh->logfile->write2log(buffer = get_errTXT( TOO_MANY_ERR ));
  rh->rc = TOO_MANY_ERR;
	cleanexit(errnum);
	}

if ((errnum >= RH_FATAL_ERROR) || (errnum < 2))
	{
  if (errnum >= RH_FATAL_ERROR) rh->rc = errnum;
	cleanexit(errnum);
	}


return;
}


void Quit::YesFunc(GtkWidget *widget, Complain *cn)
{
RevHead *rh = (RevHead *)cn->data;
cn->end_complain(widget, cn);
rh->quit->HandleError(n1, str1, str2);
}


/*******************************************************
 * Display format and template for the RevHead command *
 *******************************************************/

// Usage needs updating for Linux!

void Quit::Usage(int n, char *s1, char *s2)
{
Complain *cn =  (rh) ? (new Complain(rh->icon_pixbuf, &lightgrey, &pink, GTK_STOCK_CANCEL, GTK_STOCK_OK))
                     : (new Complain());

// This needs re-writing...  
static const char *usage =
    {
    "USAGE, Format from Shell\n"

    "revhead [[PATH<=| >]<pathname>]\n\n"
    "Default path is the current directory if no path specified.\n"
    "The PATH keyword can be upper or lower case or omitted."
    };

cout << "Entered  Usage()\n";

n1  =n;
str1=s1;
str2=s2;

cn->complain(   (int)CN_CONTINUE,
                (char *)"RevHead Usage",
                (char *)usage,
                YesFunc,
                NULL, rh);
} 


