/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    RevHead                                                   *
*Project Version: 4.3.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 16th April 2013                                   *
*                                                                           *
*Start DateStamp: 15-Sep-2008 11.16.15 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:  http://balean.net/linprogs                                          *
*                                                                           *
*Project Summary:                                                           *
* Create/maintain programming file revision headers and history             *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       tfile.cpp                                                 *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       20-Nov-2008 14.43.45 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* The temporary file module                                                 *
* 04-Jul-1999 09.18.06 -36000  0.00    David M. Balean                      *
* 20-Aug-2004 15.02.06 -36000 0.01    DMB                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/


#include "tfile.h" 

#include <sys/stat.h>
#include <stdlib.h>

TFile::TFile(void)
{
cout << "TFile:\n";
}
        
TFile::~TFile(void)
{
cout << "~TFile:\n";
}

         
int TFile::power(int n, int base)
{
  int p;

  for (p=1; n>0; --n)
    {
    p *= base;
    }

  return(p);
}

string &TFile::itoa(int n)
{
return(itoa(n, 0));
}  
  
   
string &TFile::itoa(int n, int pad)
{
char buffer[512];
char *c = &buffer[511];
int sign = (n<0) ? 1 : 0;
unsigned int p = abs(pad);

*c-- =  '\0';

do
    {
    *c-- = (n % 10) + '0'; 
    }
    while  ( (n /= 10) > 0 );


if (sign) *c = '-';
else c++;

a_val = c;

while (a_val.length() < p)
    {
    a_val = '0' + a_val;
    }
    
return(a_val);    
}


string *TFile::tfile(string *pathname, fstream *ofp, int n)
{
static string fname;
struct stat statbuf;

int max = power(10, n);
  
// i represents the suffix put into a_val
for (int i=0; i<max; i++)
    {
    itoa(i, n);
    fname = *pathname + a_val;

    if (stat(fname.c_str(), &statbuf)) // if it doesn't exist
        {
        ofp->open(fname.c_str(), ios::out | ios::binary | ios::trunc );
        if ( ofp->is_open() ) return(&fname);
        }
    }
return(NULL);
}

string *TFile::tfile(string *pathname, fstream *ofp)
{
return( tfile(pathname, ofp, 3) );
}

int TFile::do_copy(fstream *ofp, fstream *ifp)
{
if (ifp && ofp && ifp->is_open() && ofp->is_open() )
    {
    char c;
      
    while (ofp && ifp->get(c))
        ofp->put(c);
    return(1);  
    }
return(0);
}


int TFile::filecopy(string *dname, string *sname, int overwrite)
{
fstream ofile;
int rc;

fstream ifile( sname->c_str(), ios::in  | ios::binary );
if (overwrite)
    {
    ofile.open(dname->c_str(), ios::out | ios::binary | ios::trunc );
    }
else ofile.open( dname->c_str(), ios::out | ios::binary );
rc = (do_copy(&ofile, &ifile));
ofile.close();
ifile.close();
return(rc);
}

int TFile::overwritefile(string *dname, string *sname)
{
return(filecopy(dname, sname, 1));
}

int TFile::filecopy(string *dname, string *sname)
{
return(filecopy(dname, sname, 0));
}

string *TFile::copy(string *pathname, int n)
{
static string fname;
struct stat statbuf;

int max = power(n, 10);

// i represents the suffix put into a_val
for (int i=0; i<max; i++)
    {
    itoa(i, n);
    fname = *pathname + a_val;

    if (stat(fname.c_str(), &statbuf)) // if it doesn't exist
        {
         if (filecopy(&fname, pathname, 0))
            {
            return(&fname);
            }
        // otherwise try the next possible
        }
    }
return(NULL); 
}

string *TFile::copy(string *pathname)
{
return(copy(pathname, 3));
}

int TFile::append(string *dname, string *sname)
{
fstream ofp;
fstream ifp;
int rc;
  
ifp.open( sname->c_str(), ios::in  | ios::binary );
ofp.open(dname->c_str(), ios::out | ios::binary | ios::app );

rc = do_copy(&ofp, &ifp);

ifp.close();
ofp.close();

return rc;
}
